@@ -1,16 +1,14 @@
 /*
  *	machine.h
  *
- *	Definition of Machine Depended Identifier
+ *	definitions of machine depended identifier
  */
 
 #ifndef	__MACHINE_H_
 #define	__MACHINE_H_
 
-#ifdef	MSDOS
 #undef	MSDOS
-#endif
-#define	MSDOS		(DOSV || PC98 || J3100)
+#define	MSDOS			(DOSV || PC98 || J3100)
 
 #if	MSDOS
 #define	NOMULTIKANJI
@@ -23,12 +21,15 @@
 #define	BSPATHDELIM
 #define	USECRNL
 #define	CWDINPATH
+#define	DOUBLESLASH
 #define	NOTZFILEH
 #define	USETIMEH
 #define	USEUTIME
+#define	NOFLOCK
+#define	NOSYSLOG
 #define	USEMKTIME
 #define	USESTRERROR
-#define	SENSEPERSEC	20
+#define	SENSEPERSEC		20
 # if	defined (__TURBOC__) && defined (__WIN32__)
 # define	BCC32
 # endif
@@ -36,7 +37,7 @@
 # define	USERESOURCEH
 # define	SIGFNCINT
 #  ifndef	DJGPP
-#  define	DJGPP	1
+#  define	DJGPP		1
 #  else
 #  define	USEREGCOMP
 #  endif
@@ -48,10 +49,10 @@
 # define	NOUNISTDH
 # endif	/* !__GNUC__ */
 # if	!defined (__GNUC__) || (DJGPP >= 2)
-typedef unsigned int	u_int;
-typedef unsigned char	u_char;
-typedef unsigned short	u_short;
-typedef unsigned long	u_long;
+typedef unsigned int		u_int;
+typedef unsigned char		u_char;
+typedef unsigned short		u_short;
+typedef unsigned long		u_long;
 # endif
 # ifdef	__TURBOC__
 #  ifndef	BCC32
@@ -70,6 +71,9 @@ typedef long	off_t;
 # if	defined (DJGPP) || defined (BCC32)
 # define	PROTECTED_MODE
 # endif
+# if	!defined (DJGPP) || (DJGPP < 2)
+# define	NOSELECT
+# endif
 #endif	/* !MSDOS */
 
 #ifndef	NEAR
@@ -80,6 +84,7 @@ typedef long	off_t;
 #define	CODEEUC
 #define	CPP7BIT
 #define	USELEAPCNT
+#define	SOCKETLIB		"-lsocket -lnsl"
 #include <sys/param.h>
 # if	!defined (MAXHOSTNAMELEN) \
 || defined (USGr4) || defined (__svr4__) || defined (__SVR4)
@@ -97,11 +102,13 @@ typedef long	off_t;
 # define	NODNAMLEN
 # define	DNAMESIZE	1
 # define	NOTMGMTOFF
+# define	USEINSYSTMH
 # define	USESTATVFSH
 # define	USESTATVFS_T
 # define	USEMNTTABH
 # define	USEMKTIME
 # define	SIGFNCINT
+# define	USEINETPTON
 # else	/* MAXHOSTNAMELEN && !USGr4 && !__svr4__ && !__SVR4 */
 # define	BSD43
 # define	OSTYPE		"SUN_OS"
@@ -173,10 +180,10 @@ typedef long	off_t;
 #define	DECLERRLIST
 #define	USEMKDEVH
 #define	NODNAMLEN
-#define	DNAMESIZE	1
+#define	DNAMESIZE		1
 #define	NOTMGMTOFF
 #define	USESTATFSH
-#define	STATFSARGS	4
+#define	STATFSARGS		4
 #define	USERE_COMP
 #define	USEMKTIME
 #define	USESYSCONF
@@ -197,7 +204,7 @@ typedef long	off_t;
 # define	NOSTDC
 # endif
 # ifdef	__STDC_EXT__
-# define	EXTENDCCOPT		"-D_FILE_OFFSET_BITS=64"
+# define	EXTENDCCOPT	"-D_FILE_OFFSET_BITS=64"
 # endif
 /*
  *	This is a fake '#if' for some buggy HP-UX terminfo library.
@@ -213,16 +220,19 @@ typedef long	off_t;
  *
  */
 # ifndef	__HP_cc
-# define	BUGGY_HPUX		/* Maybe HP-UX 10.20 */
+# define	BUGGY_HPUX	/* Maybe HP-UX 10.20 */
 # endif
 # if	!defined (BUGGY_HPUX)
 # define	USETERMINFO
-# define	TERMCAPLIB		"-lcurses"
+# define	TERMCAPLIB	"-lcurses"
 # endif
+#define	BUGGYMAKE
 #define	USEPID_T
 #define	NOSIGLIST
 #define	NOTZFILEH
+#define	USETIMEH
 #define	NOTMGMTOFF
+#define	USEINSYSTMH
 #define	USEREGCOMP
 #define	NOTERMVAR
 #define	USEUTIME
@@ -236,6 +246,9 @@ typedef long	off_t;
 #define	USESETVBUF
 #define	SIGFNCINT
 #define	USESTRERROR
+#define	USESETSID
+#define	USESETRESUID
+#define	USESETRESGID
 #endif
 
 #if	defined (nec_ews) || defined (_nec_ews)
@@ -285,7 +298,7 @@ typedef long	off_t;
 #define	USEREGCMP
 #define	USERESOURCEH
 #define	WAITKEYPAD		720
-#define	WAITMETA		720
+#define	WAITKANJI		720
 #endif
 
 #if	defined (uniosb)
@@ -323,7 +336,8 @@ typedef long	off_t;
 && !defined (__FreeBSD__) && !defined (__NetBSD__) && !defined (__OpenBSD__)
 #define	CODEEUC
 #define	TARUSESPACE
-#define	EXTENDLIB	"-lc_r"
+#define	NOFUNCMACRO
+#define	EXTENDLIB		"-lc_r"
 #define	USEMNTINFOR
 # if	defined (SYSTYPE_BSD)
 # define	BSD43
@@ -361,6 +375,7 @@ typedef long	off_t;
 #define	USEMNTCTL
 #define	USERE_COMP
 # if	defined (_AIX41)
+# define	HAVELONGLONG
 # define	USESTATVFSH
 # define	USEMKTIME
 # define	SIGFNCINT
@@ -408,10 +423,10 @@ typedef long	off_t;
 #define	CODEEUC
 #define	TERMCAPLIB		"-ltermcap"
 #define	USESTATFSH
-#define	STATFSARGS	4
+#define	STATFSARGS		4
 #define	USESYSDIRH
 #define	NODNAMLEN
-#define	DNAMESIZE	1
+#define	DNAMESIZE		1
 #define	USERE_COMP
 #endif
 
@@ -423,7 +438,7 @@ typedef long	off_t;
 #define	REGEXPLIB		"-lgen"
 #define	USEMKDEVH
 #define	NODNAMLEN
-#define	DNAMESIZE	1
+#define	DNAMESIZE		1
 #define	NOTMGMTOFF
 #define	USESTATVFSH
 #define	USEMNTTABH
@@ -438,7 +453,7 @@ typedef long	off_t;
 #define	REGEXPLIB		"-lgen"
 #define	USEMKDEVH
 #define	NODNAMLEN
-#define	DNAMESIZE	1
+#define	DNAMESIZE		1
 #define	NOTMGMTOFF
 #define	USESTATVFSH
 #define	USEMNTTABH
@@ -465,6 +480,7 @@ typedef long	off_t;
 #define	PATHNOCASE
 #define	COMMNOCASE
 #define	USECRNL
+#define	DOUBLESLASH
 #define	USEMANLANG
 #define	BSDINSTALL
 #define	TARUSESPACE
@@ -482,6 +498,9 @@ typedef long	off_t;
 #define	USEMKTIME
 #define	DEFFDSETSIZE
 #define	SIGFNCINT
+#define	USESOCKLEN
+#define	USEINETATON
+#define	WAITKEYPAD		36
 #endif
 
 #if	defined (linux)
@@ -490,9 +509,9 @@ typedef long	off_t;
 #define	CODEEUC
 # if	defined (alpha) || defined (__alpha) || defined (__alpha__)
 /* Linux/Alpha need not support LFS and cannot use statfs(2) with LFS */
-# define	EXTENDCCOPT		"-O"
+# define	EXTENDCCOPT	"-O"
 # else
-# define	EXTENDCCOPT		"-O -D_FILE_OFFSET_BITS=64"
+# define	EXTENDCCOPT	"-O -D_FILE_OFFSET_BITS=64"
 # endif
 #define	USEMANLANG
 #define	BSDINSTALL
@@ -510,9 +529,9 @@ typedef long	off_t;
  *
  */
 # if	defined (Slackware)
-# define	TERMCAPLIB		"-ltermcap"
+# define	TERMCAPLIB	"-ltermcap"
 # else
-# define	TERMCAPLIB		"-lncurses"
+# define	TERMCAPLIB	"-lncurses"
 # endif
 #define	DECLSIGLIST
 #define	DECLERRLIST
@@ -535,6 +554,8 @@ typedef long	off_t;
 # define	USELLSEEK
 # endif
 #define	SIGFNCINT
+#define	USESOCKLEN
+#define	USEINETATON
 #endif
 
 #if	defined (__FreeBSD__) && defined (__powerpc__)
@@ -552,6 +573,7 @@ typedef long	off_t;
 #define	USESETENV
 #define	USEMKTIME
 #define	SIGFNCINT
+#define	USEINETATON
 #endif
 
 #if	defined (__FreeBSD__) && !defined (OSTYPE) && !defined (__BOW__)
@@ -576,6 +598,7 @@ typedef long	off_t;
 #define	USESETENV
 #define	USEMKTIME
 #define	SIGFNCINT
+#define	USEINETATON
 #endif
 
 #if	defined (__NetBSD__)
@@ -588,17 +611,23 @@ typedef long	off_t;
 #define	DECLSIGLIST
 #define	DECLERRLIST
 #define	USELEAPCNT
-#define	USEMOUNTH
-#define	USEMNTINFO
 #define	USEFFSTYPE
 #define	USERE_COMP
 #define	USESETENV
 #define	USEMKTIME
 #define	SIGFNCINT
+#define	USEINETATON
 #include <sys/param.h>
 # if	defined (NetBSD1_0) && (NetBSD1_0 < 1)
 # define	USEFFSIZE
 # endif
+# if	defined (__NetBSD_Version__) && (__NetBSD_Version__ >= 300000000)
+# define	USESTATVFSH
+# define	USEGETVFSTAT
+# else
+# define	USEMOUNTH
+# define	USEMNTINFO
+# endif
 #endif
 
 #if	defined (__bsdi__)
@@ -615,6 +644,7 @@ typedef long	off_t;
 #define	USERE_COMP
 #define	USESETENV
 #define	USEMKTIME
+#define	USEINETATON
 #include <sys/param.h>
 # if	!defined (BSD) || (BSD < 199306)
 # define	USEFFSIZE
@@ -642,6 +672,7 @@ typedef long	off_t;
 #define	USESETENV
 #define	USEMKTIME
 #define	SIGFNCINT
+#define	USEINETATON
 #endif
 
 #if	defined (__OpenBSD__)
@@ -661,6 +692,7 @@ typedef long	off_t;
 #define	USESETENV
 #define	USEMKTIME
 #define	SIGFNCINT
+#define	USEINETATON
 #endif
 
 #if	defined (__APPLE__) && defined (__MACH__) && !defined (OSTYPE)
@@ -681,6 +713,8 @@ typedef long	off_t;
 #define	SELECTRWONLY
 #define	USEMKTIME
 #define	SIGFNCINT
+#define	USESOCKLEN
+#define	USEINETATON
 #endif
 
 #if	defined (__386BSD__) && !defined (OSTYPE)
@@ -708,7 +742,7 @@ typedef long	off_t;
 # undef	CCCOMMAND
 # define	CCCOMMAND	"/bsd43/bin/cc"
 # endif
-#define	TERMCAPLIB	"-lcurses -ltermcap"
+#define	TERMCAPLIB		"-lcurses -ltermcap"
 #define	NOERRNO
 #define	NOFILEMODE
 #define	NOUNISTDH
@@ -772,15 +806,17 @@ typedef long	off_t;
 /* #define NOUID	;have not uid, gid */
 /* #define NODIRLOOP	;any directory structure cannot loop */
 /* #define NOSYMLINK	;have not symbolic link */
-/* #define BSPATHDELIM	;path delimtor is backspace  */
+/* #define BSPATHDELIM	;path delimtor is backspace */
 /* #define USECRNL	;use CR-NL as end of line */
 /* #define CWDINPATH	;CWD is implicitly included in command path */
+/* #define DOUBLESLASH	;pathname starting with // has some special mean */
 /* #define USEMANLANG	;man(1) directory includes LANG environment value */
 /* #define SUPPORTSJIS	;cc(1) or man(1) supports Shift JIS perfectly */
 /* #define BSDINSTALL	;install(1) with option -c is valid like BSD */
 /* #define BSDINSTCMD	;command name except "install" to install like BSD */
 /* #define TARUSESPACE	;tar(1) uses space to devide file mode from UID */
 /* #define TARFROMPAX	;tar(1) comes from pax(1) */
+/* #define BUGGYMAKE	;make(1) assumes the same timestamp as earlier */
 /* #define CPP7BIT	;cpp(1) cannot through 8bit */
 /* #define CCCOMMAND	;fullpath of suitable cc(1) */
 /* #define EXTENDCCOPT	;additional option on cc(1) */
@@ -789,10 +825,14 @@ typedef long	off_t;
 /* #define USETERMINFO	;use terminfo library instead of termcap */
 /* #define TERMCAPLIB	;library needed for termcap */
 /* #define REGEXPLIB	;library needed for regular expression */
+/* #define SOCKETLIB	;library needed for socket */
 /* #define EXTENDLIB	;library needed for the other extended */
 
 /* #define UNKNOWNFS	;use unsupported type FileSystem */
 
+/* #define NOFILEMACRO	;cc(1) cannot evaluate __FILE__ macro */
+/* #define NOFUNCMACRO	;cc(1) cannot evaluate __FUNCTION__ macro */
+/* #define NOLINEMACRO	;cc(1) cannot evaluate __LINE__ macro */
 /* #define FORCEDSTDC	;not defined __STDC__, but expect standard C */
 /* #define STRICTSTDC	;cannot allow K&R type function declaration */
 /* #define NOSTDC	;defined __STDC__, but expect traditional C */
@@ -811,24 +851,28 @@ typedef long	off_t;
 /* #define NOUNISTDH	;have not <unistd.h> */
 /* #define NOSTDLIBH	;have not <stdlib.h> */
 /* #define NOTZFILEH	;have not <tzfile.h> */
-/* #define USELEAPCNT	;struct tzhead has tzh_leapcnt as leap second */
+/* #define USELEAPCNT	;'struct tzhead' has 'tzh_leapcnt' as leap second */
 /* #define USESELECTH	;use <sys/select.h> for select() */
 /* #define USESYSDIRH	;use <sys/dir.h> for DEV_BSIZE */
 /* #define USETIMEH	;use <time.h> for 'struct tm' */
 /* #define USESTDARGH	;use <stdarg.h> for va_list */
 /* #define USEMKDEVH	;use <sys/mkdev.h> for major()/minor() */
 /* #define USEMKNODH	;use <sys/mknod.h> for major()/minor() */
+/* #define USELOCKFH	;use <sys/lockf.h> for lockf() */
 /* #define USESGTTY	;use sgtty interface */
 /* #define USETERMIO	;use termio interface */
 /* #define USETERMIOS	;use termios interface */
-/* #define HAVECLINE	;struct termios has c_line */
+/* #define HAVECLINE	;'struct termios' has 'c_line' */
+/* #define USEDEVPTMX	;use /dev/ptmx and /dev/pts/? for pseudo terminal */
 /* #define USEDIRECT	;use 'struct direct' instead of dirent */
 /* #define SYSVDIRENT	;dirent interface behaves as System V */
-/* #define NODNAMLEN	;struct dirent hasn't d_namlen */
-/* #define NODRECLEN	;struct dirent hasn't d_reclen and has d_fd instead */
-/* #define DNAMESIZE	;size of d_name in struct dirent */
+/* #define NODNAMLEN	;'struct dirent' hasn't 'd_namlen' */
+/* #define NODRECLEN	;'struct dirent' hasn't 'd_reclen' but 'd_fd' */
+/* #define DNAMESIZE	;size of 'd_name' in 'struct dirent' */
 /* #define HAVETIMEZONE	;have extern variable 'timezone' */
-/* #define NOTMGMTOFF	;struct tm hasn't tm_gmtoff */
+/* #define NOTMGMTOFF	;'struct tm' hasn't 'tm_gmtoff' */
+/* #define USEINSYSTMH	;use <netinet/in_systm.h> for <netinet/ip.h> */
+/* #define NOHADDRLIST	;'struct hostent' hasn't 'h_addr_list' */
 
 /* following 5 items are exclusive */
 /* #define USESTATVFSH	;use <sys/statvfs.h> as header of the FS status */
@@ -841,10 +885,11 @@ typedef long	off_t;
 /* #define USEFFSIZE	;'struct statfs' has 'f_fsize' as block size */
 /* #define STATFSARGS	;the number of arguments in statfs() */
 
-/* following 8 items are exclusive */
+/* following 9 items are exclusive */
 /* #define USEMNTENTH	;use <mntent.h> as header of the mount entry */
 /* #define USEMNTTABH	;use <sys/mnttab.h> as header of the mount entry */
 /* #define USEGETFSSTAT	;use getfsstat() to get the mount entry */
+/* #define USEGETVFSTAT	;use getvfsstat() to get the mount entry */
 /* #define USEMNTCTL	;use mntctl() to get the mount entry */
 /* #define USEMNTINFOR	;use getmntinfo_r() to get the mount entry */
 /* #define USEMNTINFO	;use getmntinfo() to get the mount entry */
@@ -869,6 +914,10 @@ typedef long	off_t;
 /* #define NOTERMVAR	;have not termcap variables such as PC, ospeed, etc. */
 /* #define USEUTIME	;use utime() instead of utimes() */
 /* #define USEGETWD	;use getwd() instead of getcwd() */
+/* #define USEFCNTLOCK	;use fcntl() lock instead of flock() */
+/* #define USELOCKF	;use lockf() instead of flock() */
+/* #define NOFLOCK	;have neither flock() nor lockf() */
+/* #define NOSYSLOG	;have not syslog() */
 /* #define USETIMELOCAL	;have timelocal() as inverse of localtime() */
 /* #define USEMKTIME	;use mktime() instead of timelocal() */
 /* #define USESYSCONF	;use sysconf() for getting system configuration */
@@ -888,10 +937,20 @@ typedef long	off_t;
 /* #define SIGFNCINT	;the 2nd argument function of signal() needs int */
 /* #define USESTRERROR	;use strerror() instead of sys_errlist[] */
 /* #define GETTODARGS	;the number of arguments in gettimeofday() */
+/* #define USESETSID	;use setsid() to set session ID */
+/* #define USEMMAP	;use mmap() to map files into memory */
+/* #define USESOCKLEN	;use socklen_t for bind()/connect()/accept() */
+/* #define USEINETATON	;use inet_aton() instead of inet_addr() */
+/* #define USEINETPTON	;use inet_pton() instead of inet_addr() */
+/* #define USESETREUID	;use setreuid() to set effective user ID */
+/* #define USESETRESUID	;use setresuid() to set effective user ID */
+/* #define USESETREGID	;use setregid() to set effective group ID */
+/* #define USESETRESGID	;use setresgid() to set effective group ID */
+/* #define USEGETGROUPS	;use getgroups() to get group access list */
 
 /* #define SENSEPERSEC	;ratio of key sense per 1 second */
 /* #define WAITKEYPAD	;interval to wait after getting input of ESC [ms] */
-/* #define WAITMETA	;interval to wait after getting input of META [ms] */
+/* #define WAITKANJI	;interval to wait after getting input of Kanji [ms] */
 
 
 /*                             */
@@ -899,11 +958,11 @@ typedef long	off_t;
 /*                             */
 
 #ifdef	BSPATHDELIM
-#define	_SC_	'\\'
-#define	_SS_	"\\"
+#define	_SC_			'\\'
+#define	_SS_			"\\"
 #else
-#define	_SC_	'/'
-#define	_SS_	"/"
+#define	_SC_			'/'
+#define	_SS_			"/"
 #endif
 
 #if	defined (BSD43) || defined (BSD44)
@@ -929,10 +988,13 @@ typedef long	off_t;
 #define	USESETPGID
 #define	USESETVBUF
 #define	USESTRERROR
+#define	USESETSID
+#define	USEMMAP
 #endif
 
 #ifdef	SVR4
 #define	SYSV
+#define	USESOCKLEN
 #endif
 
 #ifdef	SYSV
@@ -949,24 +1011,43 @@ typedef long	off_t;
 # endif
 #endif
 
+#ifndef	BSD4
+#define	USEDEVPTMX
+#endif
+
+#if	defined (POSIX) || defined (SYSV)
+#define	USEFCNTLOCK
+#endif
+
+#if	defined (BSD4)
+#define	USEINSYSTMH
+#endif
+
+#if	defined (BSD4) && !defined (BSD43) && !defined (BSD44)
+#define	NOHADDRLIST
+#endif
+
+#if	defined (POSIX) || defined (BSD4)
+#define	USEGETGROUPS
+#endif
+
+#if	!defined (__GNUC__) && !defined (__attribute__)
+#define	__attribute__(x)
+#endif
+
 /*                                 */
 /* Eval configurations by Configur */
 /*                                 */
 
 #include "config.h"
 
-
 #ifdef	USETERMIOS
-# ifdef	USETERMIO
-# undef	USETERMIO
-# endif
+#undef	USETERMIO
 #endif
-
 #if	defined (USETERMIOS) || defined (USETERMIO)
-# ifdef	USESGTTY
-# undef	USESGTTY
-# endif
-#else
+#undef	USESGTTY
+#endif
+#if	!defined (USETERMIOS) && !defined (USETERMIO) && !defined (USESGTTY)
 #define	USESGTTY
 #endif
 
@@ -988,35 +1069,35 @@ typedef long	off_t;
 #endif
 
 #if	defined (FORCEDSTDC) && !defined (STRICTSTDC)
-#define	__P_(args)	args
+#define	__P_(args)		args
 #else
-#define	__P_(args)	()
+#define	__P_(args)		()
 #endif
 
 #if	defined (FORCEDSTDC) && !defined (NOCONST)
-#define	CONST		const
+#define	CONST			const
 #else
 #define	CONST
 #endif
 
 #ifdef	FORCEDSTDC
-#define	ALLOC_T		size_t
-#define	VOID_A		void
+#define	ALLOC_T			size_t
+#define	VOID_A			void
 #else
-#define	ALLOC_T		unsigned
+#define	ALLOC_T			u_int
 #define	VOID_A
 #endif
 
 #ifdef	NOVOID
 #define	VOID
-#define	VOID_T	int
-#define	VOID_P	char *
+#define	VOID_T			int
+#define	VOID_P			char *
 #define	VOID_C
 #else
-#define	VOID	void
-#define	VOID_T	void
-#define	VOID_P	void *
-#define	VOID_C	(void)
+#define	VOID			void
+#define	VOID_T			void
+#define	VOID_P			void *
+#define	VOID_C			(void)
 #endif
 
 #if	defined (USEREGCMP) && !defined (REGEXPLIB)
@@ -1038,14 +1119,17 @@ typedef long	off_t;
 #endif
 #ifndef	TERMCAPLIB
 # ifdef	USETERMINFO
-# define	TERMCAPLIB		"-lcurses"
+# define	TERMCAPLIB	"-lcurses"
 # else
-# define	TERMCAPLIB		"-ltermlib"
+# define	TERMCAPLIB	"-ltermlib"
 # endif
 #endif
 #ifndef	REGEXPLIB
 #define	REGEXPLIB		""
 #endif
+#ifndef	SOCKETLIB
+#define	SOCKETLIB		""
+#endif
 #ifndef	EXTENDLIB
 #define	EXTENDLIB		""
 #endif
@@ -1054,29 +1138,24 @@ typedef long	off_t;
 #endif
 
 #if	defined (USESTATVFSH)
-# ifdef	USESTATFSH
-# undef	USESTATFSH
-# endif
+#undef	USESTATFSH
 #endif
 
 #if	defined (USESTATVFSH) || defined (USESTATFSH)
-# ifdef	USEMOUNTH
-# undef	USEMOUNTH
-# endif
+#undef	USEMOUNTH
 #endif
 
 #if	defined (USESTATVFSH) || defined (USESTATFSH) || defined (USEMOUNTH)
-# ifdef	USEFSDATA
-# undef	USEFSDATA
-# endif
+#undef	USEFSDATA
 #endif
 
-#if	defined (USESTATVFSH) || defined (USESTATFSH) || defined (USEMOUNTH) \
-|| defined (USEFSDATA)
-# ifdef	USEVFSH
-# undef	USEVFSH
-# endif
-#else
+#if	defined (USESTATVFSH) || defined (USESTATFSH) \
+|| defined (USEMOUNTH) || defined (USEFSDATA)
+#undef	USEVFSH
+#endif
+
+#if	!defined (USESTATVFSH) && !defined (USESTATFSH) \
+&& !defined (USEMOUNTH) && !defined (USEFSDATA) && !defined (USEVFSH)
 #define	USEVFSH
 #endif
 
@@ -1088,61 +1167,66 @@ typedef long	off_t;
 
 
 #if	defined (USEMNTTABH)
-# ifdef	USEGETFSSTAT
-# undef	USEGETFSSTAT
-# endif
+#undef	USEGETFSSTAT
 #endif
 
 #if	defined (USEMNTTABH) || defined (USEGETFSSTAT)
-# ifdef	USEMNTCTL
-# undef	USEMNTCTL
-# endif
+#undef	USEGETVFSTAT
 #endif
 
-#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) || defined (USEMNTCTL)
-# ifdef	USEMNTINFOR
-# undef	USEMNTINFOR
-# endif
+#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) \
+|| defined (USEGETVFSTAT)
+#undef	USEMNTCTL
 #endif
 
-#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) || defined (USEMNTCTL) \
-|| defined (USEMNTINFOR)
-# ifdef	USEMNTINFO
-# undef	USEMNTINFO
-# endif
+#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) \
+|| defined (USEGETVFSTAT) || defined (USEMNTCTL)
+#undef	USEMNTINFOR
 #endif
 
-#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) || defined (USEMNTCTL) \
-|| defined (USEMNTINFOR) || defined (USEMNTINFO)
-# ifdef	USEGETMNT
-# undef	USEGETMNT
-# endif
+#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) \
+|| defined (USEGETVFSTAT) || defined (USEMNTCTL) || defined (USEMNTINFOR)
+#undef	USEMNTINFO
 #endif
 
-#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) || defined (USEMNTCTL) \
-|| defined (USEMNTINFOR) || defined (USEMNTINFO) || defined (USEGETMNT)
-# ifdef	USEGETFSENT
-# undef	USEGETFSENT
-# endif
+#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) \
+|| defined (USEGETVFSTAT) || defined (USEMNTCTL) || defined (USEMNTINFOR) \
+|| defined (USEMNTINFO)
+#undef	USEGETMNT
 #endif
 
-#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) || defined (USEMNTCTL) \
-|| defined (USEMNTINFOR) || defined (USEMNTINFO) || defined (USEGETMNT) \
-|| defined (USEGETFSENT)
-# ifdef	USEMNTENTH
-# undef	USEMNTENTH
-# endif
-#else
+#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) \
+|| defined (USEGETVFSTAT) || defined (USEMNTCTL) || defined (USEMNTINFOR) \
+|| defined (USEMNTINFO) || defined (USEGETMNT)
+#undef	USEGETFSENT
+#endif
+
+#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) \
+|| defined (USEGETVFSTAT) || defined (USEMNTCTL) || defined (USEMNTINFOR) \
+|| defined (USEMNTINFO) || defined (USEGETMNT) || defined (USEGETFSENT)
+#undef	USEMNTENTH
+#endif
+
+#if	!defined (USEMNTTABH) && !defined (USEGETFSSTAT) \
+&& !defined (USEGETVFSTAT) && !defined (USEMNTCTL) && !defined (USEMNTINFOR) \
+&& !defined (USEMNTINFO) && !defined (USEGETMNT) && !defined (USEGETFSENT) \
+&& !defined (USEMNTENTH)
 #define	USEMNTENTH
 #endif
 
+#if	MSDOS
+#undef	USEVFSH
+#undef	USEMNTENTH
+#endif
+
+
 #if	!defined (USEGETRUSAGE) && !defined (USETIMES) \
 && defined (USERESOURCEH)
 #define	USEGETRUSAGE
 #endif
 
 #ifndef	GETTODARGS
-#define	GETTODARGS	2
+#define	GETTODARGS		2
 #endif
 
-#endif	/* __MACHINE_H_ */
+#endif	/* !__MACHINE_H_ */
