@@ -1,29 +1,57 @@
 /*
  *	parse.c
  *
- *	Commandline Parser
+ *	command line parser
  */
 
+#ifdef	FD
 #include "fd.h"
-#include "func.h"
+#include "pathname.h"
+#include "term.h"
+#include "types.h"
+#include "kconv.h"
+#else
+#include "headers.h"
+#include "depend.h"
+#include "printf.h"
+#include "kctype.h"
+#include "string.h"
+#include "malloc.h"
+#endif
 
+#include "sysemu.h"
+#include "unixemu.h"
+#include "realpath.h"
+#include "parse.h"
+
+#ifdef	DEP_ORIGSHELL
+#include "system.h"
+#endif
 #ifdef	USEUNAME
 #include <sys/utsname.h>
 #endif
 
+#ifdef	FD
+# ifndef	DEP_ORIGSHELL
+extern int setenv2 __P_((CONST char *, CONST char *, int));
+# endif
+extern int underhome __P_((char *));
+#endif	/* !FD */
+
+#ifdef	FD
 extern char fullpath[];
 extern short histno[];
 extern int physical_path;
-
-static int NEAR asc2int __P_((int));
-#ifdef	_NOORIGSHELL
-static char *NEAR strtkbrk __P_((char *, char *, int));
-static char *NEAR geteostr __P_((char **));
-#else
-#include "system.h"
 #endif
 
-strtable keyidentlist[] = {
+static int NEAR asc2int __P_((int));
+#if	defined (FD) && !defined (DEP_ORIGSHELL)
+static char *NEAR strtkbrk __P_((CONST char *, CONST char *, int));
+static char *NEAR geteostr __P_((CONST char **));
+#endif
+
+#ifdef	FD
+CONST strtable keyidentlist[] = {
 	{K_DOWN,	"DOWN"},
 	{K_UP,		"UP"},
 	{K_LEFT,	"LEFT"},
@@ -92,7 +120,8 @@ strtable keyidentlist[] = {
 	{K_HELP,	"HELP"},
 	{0,		NULL}
 };
-#define	KEYIDENTSIZ	((int)(sizeof(keyidentlist) / sizeof(strtable)) - 1)
+#define	KEYIDENTSIZ		(arraysize(keyidentlist) - 1)
+#endif	/* FD */
 
 static CONST char escapechar[] = "abefnrtv";
 static CONST char escapevalue[] = {
@@ -101,10 +130,11 @@ static CONST char escapevalue[] = {
 
 
 char *skipspace(cp)
-char *cp;
+CONST char *cp;
 {
-	while (*cp == ' ' || *cp == '\t') cp++;
-	return(cp);
+	while (isblank2(*cp)) cp++;
+
+	return((char *)cp);
 }
 
 static int NEAR asc2int(c)
@@ -114,50 +144,54 @@ int c;
 	else if (isdigit2(c)) return(c - '0');
 	else if (islower2(c)) return(c - 'a' + 10);
 	else if (isupper2(c)) return(c - 'A' + 10);
-	else return(-1);
+
+	return(-1);
 }
 
 #ifdef	USESTDARGH
 /*VARARGS2*/
-char *sscanf2(char *s, CONST char *fmt, ...)
+char *sscanf2(CONST char *s, CONST char *fmt, ...)
 #else
 /*VARARGS2*/
 char *sscanf2(s, fmt, va_alist)
-char *s;
-CONST char *fmt;
+CONST char *s, *fmt;
 va_dcl
 #endif
 {
+#ifndef	HAVELONGLONG
+	char *buf;
+	u_long_t tmp;
+	int hi;
+#endif
 	va_list args;
-	char *cp;
+	CONST char *cp;
 	long_t n;
 	u_long_t u, mask;
 	int i, c, len, base, width, flags;
 
-#ifdef	USESTDARGH
-	va_start(args, fmt);
-#else
-	va_start(args);
-#endif
-
 	if (!s || !fmt) return(NULL);
+	VA_START(args, fmt);
+
 	for (i = 0; fmt[i]; i++) {
 		if (fmt[i] != '%') {
-			if (*(s++) != fmt[i]) return(NULL);
+			if (*(s++) != fmt[i]) {
+				s = NULL;
+				break;
+			}
 			continue;
 		}
 
 		i++;
 		flags = 0;
 		for (; fmt[i]; i++) {
-			if (!(cp = strchr(printfflagchar, fmt[i]))) break;
+			if (!(cp = strchr2(printfflagchar, fmt[i]))) break;
 			flags |= printfflag[cp - printfflagchar];
 		}
 		width = getnum(fmt, &i);
 
 		len = sizeof(int);
 		for (; fmt[i]; i++) {
-			if (!(cp = strchr(printfsizechar, fmt[i]))) break;
+			if (!(cp = strchr2(printfsizechar, fmt[i]))) break;
 			len = printfsize[cp - printfsizechar];
 		}
 		if (fmt[i] == '*') {
@@ -184,22 +218,26 @@ va_dcl
 				base = 16;
 				break;
 			case 'c':
-				if (*(s++) != va_arg(args, int)) return(NULL);
+				if (*(s++) != va_arg(args, int)) s = NULL;
 				break;
 			case '$':
-				if (*s) return(NULL);
+				if (*s) s = NULL;
 				break;
 			default:
-				if (*(s++) != fmt[i]) return(NULL);
+				if (*(s++) != fmt[i]) s = NULL;
 				break;
 		}
+		if (!s) break;
 		if (!base) continue;
 
 		cp = s;
 		n = (long_t)0;
 		u = (u_long_t)0;
 		if (*s == '-') {
-			if (!(flags & VF_MINUS)) return(NULL);
+			if (!(flags & VF_MINUS)) {
+				s = NULL;
+				break;
+			}
 			for (s++; *s; s++) {
 #ifdef	LSI_C
 			/* for buggy LSI-C */
@@ -218,7 +256,7 @@ va_dcl
 					n = MINTYPE(long_t);
 				else n = n * base - c;
 			}
-			if ((flags & VF_UNSIGNED) && n) return(NULL);
+			if ((flags & VF_UNSIGNED) && n) s = NULL;
 		}
 		else if (flags & VF_UNSIGNED) {
 			for (; *s; s++) {
@@ -231,7 +269,7 @@ va_dcl
 					u = MAXTYPE(u_long_t);
 				else u = u * base + c;
 			}
-			if ((flags & VF_PLUS) && !u) return(NULL);
+			if ((flags & VF_PLUS) && !u) s = NULL;
 		}
 		else {
 			for (; *s; s++) {
@@ -244,46 +282,91 @@ va_dcl
 					n = MAXTYPE(long_t);
 				else n = n * base + c;
 			}
-			if ((flags & VF_PLUS) && !n) return(NULL);
+			if ((flags & VF_PLUS) && !n) s = NULL;
+		}
+		if (!s || s <= cp || ((flags & VF_ZERO) && width > 0)) {
+			s = NULL;
+			break;
 		}
-		if (s <= cp) return(NULL);
-		if ((flags & VF_ZERO) && width > 0) return(NULL);
 
-		mask = (MAXUTYPE(u_long_t)
-			>> ((sizeof(long_t) - len) * BITSPERBYTE));
+		mask = MAXUTYPE(u_long_t);
+		if (len < (int)sizeof(u_long_t))
+			mask >>= ((int)sizeof(u_long_t) - len) * BITSPERBYTE;
 		if (flags & VF_UNSIGNED) {
-			if (u & ~mask) return(NULL);
+			if (u & ~mask) {
+				u = mask;
+				if (flags & VF_STRICTWIDTH) {
+					s = NULL;
+					break;
+				}
+			}
 		}
 		else {
 			mask >>= 1;
 			if (n >= 0) {
-				if ((u_long_t)n & ~mask) return(NULL);
+				if ((u_long_t)n & ~mask) {
+					n = mask;
+					if (flags & VF_STRICTWIDTH) s = NULL;
+				}
 			}
-			else if (((u_long_t)n & ~mask) != ~mask) return(NULL);
+			else if (((u_long_t)n & ~mask) != ~mask) {
+				n = ~mask;
+				if (flags & VF_STRICTWIDTH) s = NULL;
+			}
+			if (!s) break;
 			memcpy(&u, &n, sizeof(u));
 		}
 
-		if (len == sizeof(u_long_t))
+#ifndef	HAVELONGLONG
+		if (len > (int)sizeof(u_long_t)) {
+			hi = 0;
+			if (!(flags & VF_UNSIGNED)) {
+				mask = (MAXUTYPE(u_long_t) >> 1);
+				if (u & ~mask) hi = MAXUTYPE(char);
+			}
+			buf = va_arg(args, char *);
+			memset(buf, hi, len);
+
+			tmp = 0x5a;
+			cp = (char *)(&tmp);
+			if (*cp != 0x5a) buf += len - sizeof(u_long_t);
+			memcpy(buf, (char *)(&u), sizeof(u));
+		}
+		else
+#endif	/* !HAVELONGLONG */
+		if (len == (int)sizeof(u_long_t))
 			*(va_arg(args, u_long_t *)) = u;
 #ifdef	HAVELONGLONG
-		else if (len == sizeof(u_long)) *(va_arg(args, u_long *)) = u;
+		else if (len == (int)sizeof(u_long))
+			*(va_arg(args, u_long *)) = u;
 #endif
-		else if (len == sizeof(u_short))
+		else if (len == (int)sizeof(u_short))
 			*(va_arg(args, u_short *)) = u;
-		else if (len == sizeof(u_char)) *(va_arg(args, u_char *)) = u;
+		else if (len == (int)sizeof(u_char))
+			*(va_arg(args, u_char *)) = u;
 		else *(va_arg(args, u_int *)) = u;
 	}
 	va_end(args);
 
-	return(s);
+	return((char *)s);
 }
 
-#ifdef	_NOORIGSHELL
+int atoi2(s)
+CONST char *s;
+{
+	int n;
+
+	if (!sscanf2(s, "%<d%$", &n)) return(-1);
+
+	return(n);
+}
+
+#if	defined (FD) && !defined (DEP_ORIGSHELL)
 static char *NEAR strtkbrk(s, c, evaldq)
-char *s, *c;
+CONST char *s, *c;
 int evaldq;
 {
-	char *cp;
+	CONST char *cp;
 	int pc, quote;
 
 	for (cp = s, quote = '\0'; *cp; cp++) {
@@ -297,49 +380,55 @@ int evaldq;
 		else if (pc == PC_DQUOTE) {
 			if (!evaldq) continue;
 		}
-		else if (pc == PC_WORD) {
+		else if (pc == PC_WCHAR) {
 			cp++;
 			continue;
 		}
-		else if (pc == PC_META) {
+		else if (pc == PC_ESCAPE) {
 			cp++;
-			if (*cp == PMETA && strchr(c, *cp)) return(cp - 1);
+			if (*cp == PESCAPE && strchr2(c, *cp))
+				return((char *)&(cp[-1]));
 			continue;
 		}
 
-		if (strchr(c, *cp)) return(cp);
+		if (strchr2(c, *cp)) return((char *)cp);
 	}
+
 	return(NULL);
 }
 
 char *strtkchr(s, c, evaldq)
-char *s;
+CONST char *s;
 int c, evaldq;
 {
 	char tmp[2];
 
 	tmp[0] = c;
 	tmp[1] = '\0';
+
 	return(strtkbrk(s, tmp, evaldq));
 }
 
 static char *NEAR geteostr(strp)
-char **strp;
+CONST char **strp;
 {
-	char *cp, *tmp;
+	CONST char *cp;
+	char *tmp;
 	int len;
 
 	cp = *strp;
 	if ((tmp = strtkbrk(*strp, " \t", 0))) len = tmp - *strp;
 	else len = strlen(*strp);
 	*strp += len;
+
 	return(strndup2(cp, len));
 }
 
 int getargs(s, argvp)
-char *s, ***argvp;
+CONST char *s;
+char ***argvp;
 {
-	char *cp;
+	CONST char *cp;
 	int i;
 
 	*argvp = (char **)malloc2(1 * sizeof(char *));
@@ -350,60 +439,65 @@ char *s, ***argvp;
 		cp = skipspace(cp);
 	}
 	(*argvp)[i] = NULL;
+
 	return(i);
 }
 
 char *gettoken(s)
-char *s;
+CONST char *s;
 {
 	if (!isidentchar(*s)) return(NULL);
-	for (s++; isidentchar2(*s); s++);
-	return(s);
+	for (s++; isidentchar2(*s); s++) /*EMPTY*/;
+
+	return((char *)s);
 }
 
 char *getenvval(argcp, argv)
 int *argcp;
-char *argv[];
+char *CONST argv[];
 {
 	char *cp;
 	int i;
 
-	if (*argcp <= 0) return((char *)-1);
+	if (*argcp <= 0) return(vnullstr);
 	i = 0;
-	if (!isidentchar(argv[i][0])) return((char *)-1);
+	if (!isidentchar(argv[i][0])) return(vnullstr);
 
 	for (cp = &(argv[i][1]); *cp; cp++) if (!isidentchar2(*cp)) break;
 	cp = skipspace(cp);
 	if (!*cp) {
-		if (++i >= *argcp) return((char *)-1);
+		if (++i >= *argcp) return(vnullstr);
 		cp = skipspace(argv[i]);
 	}
-	if (*cp != '=') return((char *)-1);
+	if (*cp != '=') return(vnullstr);
 	*(cp++) = '\0';
 
 	if (!*cp) {
-		if (++i >= *argcp) return((char *)NULL);
+		if (++i >= *argcp) return(NULL);
 		cp = argv[i];
 	}
 	*argcp = i + 1;
+
 	return(evalpath(strdup2(cp), EA_NOUNIQDELIM));
 }
 
 char *evalcomstr(path, delim)
-char *path, *delim;
+CONST char *path, *delim;
 {
-# ifndef	_NOKANJIFCONV
+# ifdef	DEP_FILECONV
 	char *tmp;
 # endif
-	char *cp, *next, *epath;
+	CONST char *cp, *next;
+	char *new, *epath;
 	int i, len, size;
 
-	epath = next = NULL;
+	next = epath = NULL;
 	size = 0;
 	for (cp = path; cp && *cp; cp = next) {
 		if ((next = strtkbrk(cp, delim, 0))) {
 			len = next - cp;
-			for (i = 1; next[i] && strchr(delim, next[i]); i++);
+			for (i = 1; next[i]; i++)
+				if (!strchr2(delim, next[i])) break;
 		}
 		else {
 			len = strlen(cp);
@@ -411,142 +505,155 @@ char *path, *delim;
 		}
 		next = cp + len;
 		if (len) {
-			cp = _evalpath(cp, next, EA_NOEVALQ | EA_NOUNIQDELIM);
-# ifndef	_NOKANJIFCONV
-			tmp = newkanjiconv(cp, DEFCODE, getkcode(cp), L_FNAME);
-			if (cp != tmp) free(cp);
-			cp = tmp;
+			new = _evalpath(cp, next, EA_NOEVALQ | EA_NOUNIQDELIM);
+# ifdef	DEP_FILECONV
+			tmp = newkanjiconv(new,
+				DEFCODE, getkcode(new), L_FNAME);
+			if (new != tmp) {
+				free2(new);
+				new = tmp;
+			}
 # endif
+			cp = new;
 			len = strlen(cp);
 		}
+# ifdef	FAKEUNINIT
+		else new = NULL;	/* fake for -Wuninitialized */
+# endif
 
 		epath = realloc2(epath, size + len + i + 1);
 		if (len) {
-			strcpy(&(epath[size]), cp);
-			free(cp);
+			strcpy2(&(epath[size]), cp);
+			free2(new);
 			size += len;
 		}
 		if (i) {
-			strncpy(&(epath[size]), next, i);
+			strncpy2(&(epath[size]), next, i);
 			size += i;
 			next += i;
 		}
 	}
 
-	if (!epath) return(strdup2(""));
+	if (!epath) return(strdup2(nullstr));
 	epath[size] = '\0';
+
 	return(epath);
 }
-#endif	/* _NOORIGSHELL */
+#endif	/* FD && !DEP_ORIGSHELL */
 
 char *evalpaths(paths, delim)
-char *paths;
+CONST char *paths;
 int delim;
 {
-	char *cp, *tmp, *next, *epath, buf[MAXPATHLEN];
+	CONST char *cp, *next;
+	char *new, *epath, buf[MAXPATHLEN];
 	int len, size;
 
-	epath = next = NULL;
+	next = epath = NULL;
 	size = 0;
 	for (cp = paths; cp; cp = next) {
-#ifdef	_USEDOSPATH
-		if (_dospath(cp)) next = strchr(&(cp[2]), delim);
+#ifdef	DEP_DOSPATH
+		if (_dospath(cp)) next = strchr2(&(cp[2]), delim);
 		else
 #endif
-		next = strchr(cp, delim);
+		next = strchr2(cp, delim);
 		len = (next) ? (next++) - cp : strlen(cp);
 		if (len) {
-			tmp = _evalpath(cp, cp + len, 0);
-			if (!isrootdir(cp)) cp = tmp;
-			else cp = realpath2(tmp, buf, 1);
+			new = _evalpath(cp, &(cp[len]), 0);
+			cp = (isrootdir(cp))
+				? realpath2(new, buf, RLP_READLINK) : new;
 			len = strlen(cp);
 		}
 #ifdef	FAKEUNINIT
-		else tmp = NULL;	/* fake for -Wuninitialized */
+		else new = NULL;	/* fake for -Wuninitialized */
 #endif
 		epath = realloc2(epath, size + len + 1 + 1);
 		if (len) {
-			strcpy(&(epath[size]), cp);
-			free(tmp);
+			strcpy2(&(epath[size]), cp);
+			free2(new);
 		}
 		size += len;
 		if (next) epath[size++] = delim;
 	}
 
-	if (!epath) return(strdup2(""));
+	if (!epath) return(strdup2(nullstr));
 	epath[size] = '\0';
+
 	return(epath);
 }
 
-#if	!MSDOS || !defined (_NOORIGSHELL)
+#if	!MSDOS || !defined (FD) || defined (DEP_ORIGSHELL)
 char *killmeta(name)
-char *name;
+CONST char *name;
 {
-	char *cp, *buf;
+	CONST char *cp;
+	char *buf, *new;
 	int i;
 
 	buf = malloc2(strlen(name) * 2 + 2 + 1);
 	*buf = (*name == '~') ? '"' : '\0';
 	for (cp = name, i = 1; *cp; cp++, i++) {
-		if (iskanji1(cp, 0)) buf[i++] = *(cp++);
-# ifdef	CODEEUC
-		else if (isekana(cp, 0)) buf[i++] = *(cp++);
-# endif
-		else if (*cp == PMETA) {
+		if (iswchar(cp, 0)) buf[i++] = *(cp++);
+		else if (*cp == PESCAPE) {
 			*buf = '"';
-			if (strchr(DQ_METACHAR, *(cp + 1))) buf[i++] = PMETA;
+			if (strchr2(DQ_METACHAR, *(cp + 1)))
+				buf[i++] = PESCAPE;
 		}
-		else if (strchr(METACHAR, *cp)) {
+		else if (strchr2(METACHAR, *cp)) {
 			*buf = '"';
-			if (strchr(DQ_METACHAR, *cp)) buf[i++] = PMETA;
+			if (strchr2(DQ_METACHAR, *cp)) buf[i++] = PESCAPE;
 		}
 		buf[i] = *cp;
 	}
 	if (*(cp = buf)) buf[i++] = *cp;
 	else cp++;
 	buf[i] = '\0';
-	cp = strdup2(cp);
-	free(buf);
-	return(cp);
-}
-#endif	/* !MSDOS || !_NOORIGSHELL */
+	new = strdup2(cp);
+	free2(buf);
 
-#ifdef	_NOORIGSHELL
+	return(new);
+}
+#endif	/* !MSDOS || !FD || DEP_ORIGSHELL */
+
+#if	defined (FD) && !defined (DEP_ORIGSHELL)
 VOID adjustpath(VOID_A)
 {
 	char *cp, *path;
 
-	if (!(cp = getconstvar("PATH"))) return;
+	if (!(cp = (char *)getenv(ENVPATH))) return;
 
 	path = evalpaths(cp, PATHDELIM);
-	if (strpathcmp(path, cp)) setenv2("PATH", path, 1);
-	free(path);
+	if (strpathcmp(path, cp)) setenv2(ENVPATH, path, 1);
+	free2(path);
 }
-#endif	/* _NOORIGSHELL */
+#endif	/* FD && !DEP_ORIGSHELL */
 
+#ifdef	FD
 char *includepath(path, plist)
-char *path, *plist;
+CONST char *path, *plist;
 {
-	char *cp, *tmp, *next;
+	CONST char *cp, *next;
+	char *tmp;
 	int len;
 
 	if (!plist || !*plist) return(NULL);
 	next = plist;
 	for (cp = next; cp && *cp; cp = next) {
-#ifdef	_USEDOSPATH
-		if (_dospath(cp)) next = strchr(&(cp[2]), PATHDELIM);
-		else
-#endif
-		next = strchr(cp, PATHDELIM);
+# ifdef	DEP_DOSPATH
+		if (_dospath(cp)) next += 2;
+# endif
+		next = strchr2(next, PATHDELIM);
 		len = (next) ? (next++) - cp : strlen(cp);
 		if ((tmp = underpath(path, cp, len))) return(tmp);
 	}
+
 	return(NULL);
 }
+#endif	/* FD */
 
-#if	(FD < 2) && !defined (_NOARCHIVE)
+#if	defined (OLDPARSE) && !defined (_NOARCHIVE)
 char *getrange(cp, delim, fp, dp, wp)
-char *cp;
+CONST char *cp;
 int delim;
 u_char *fp, *dp, *wp;
 {
@@ -556,7 +663,7 @@ u_char *fp, *dp, *wp;
 	*fp = *dp = *wp = 0;
 
 	if (!(cp = sscanf2(cp, "%c%Cu", delim, &c))) return(NULL);
-	*fp = (c) ? c - 1 : 255;
+	*fp = (c) ? c - 1 : FLD_NONE;
 
 	if (*cp == '[') {
 		if (!(cp = sscanf2(++cp, "%Cu]", &c))) return(NULL);
@@ -577,12 +684,14 @@ u_char *fp, *dp, *wp;
 		else if (*cp && *cp != ',' && *cp != ':') *wp = *(cp++) % 128;
 		else return(NULL);
 	}
-	return(cp);
+
+	return((char *)cp);
 }
-#endif	/* (FD < 2) && !_NOARCHIVE */
+#endif	/* OLDPARSE && !_NOARCHIVE */
 
 int evalprompt(bufp, prompt)
-char **bufp, *prompt;
+char **bufp;
+CONST char *prompt;
 {
 #ifndef	NOUID
 	uidtable *up;
@@ -590,14 +699,16 @@ char **bufp, *prompt;
 #if	!MSDOS && defined (USEUNAME)
 	struct utsname uts;
 #endif
-	char *cp, *tmp, line[MAXPATHLEN];
+	char *cp, *tmp, *new, line[MAXPATHLEN];
 	ALLOC_T size;
 	int i, j, k, len, unprint;
 
-#ifdef	_NOORIGSHELL
-	prompt = evalpath(strdup2(prompt), EA_NOUNIQDELIM);
+	cp = strdup2(prompt);
+#if	defined (FD) && !defined (DEP_ORIGSHELL)
+	prompt = new = evalpath(cp, EA_NOUNIQDELIM);
 #else
-	prompt = evalvararg(prompt, '\0', EA_BACKQ | EA_KEEPMETA, 0);
+	prompt = new = evalvararg(cp, EA_BACKQ | EA_KEEPESCAPE, 0);
+	free2(cp);
 #endif
 	unprint = 0;
 #ifdef	FAKEUNINIT
@@ -607,25 +718,24 @@ char **bufp, *prompt;
 	for (i = j = len = 0; prompt[i]; i++) {
 		cp = NULL;
 		line[0] = '\0';
-		if (prompt[i] != META) {
+		if (prompt[i] != ESCAPE) {
 			k = 0;
 			line[k++] = prompt[i];
-			if (iskanji1(prompt, i)) line[k++] = prompt[++i];
-#ifdef	CODEEUC
-			else if (isekana(prompt, i)) line[k++] = prompt[++i];
-#endif
+			if (iswchar(prompt, i)) line[k++] = prompt[++i];
 			line[k] = '\0';
 		}
 		else switch (prompt[++i]) {
 			case '\0':
 				i--;
-				*line = META;
+				*line = ESCAPE;
 				line[1] = '\0';
 				break;
+#ifdef	FD
 			case '!':
 				snprintf2(line, sizeof(line), "%d",
 					(int)histno[0] + 1);
 				break;
+#endif
 #ifndef	NOUID
 			case 'u':
 				if ((up = finduid(getuid(), NULL)))
@@ -637,39 +747,50 @@ char **bufp, *prompt;
 			case 'H':
 # ifdef	USEUNAME
 				uname(&uts);
-				strcpy(line, uts.nodename);
+				strcpy2(line, uts.nodename);
 # else
 				gethostname(line, MAXPATHLEN);
 # endif
 				if (prompt[i] == 'h'
-				&& (tmp = strchr(line, '.')))
+				&& (tmp = strchr2(line, '.')))
 					*tmp = '\0';
 				break;
 			case '$':
 				*line = (getuid()) ? '$' : '#';
 				line[1] = '\0';
 				break;
+#endif	/* !MSDOS */
+			case '~':
+#ifdef	FD
+				if (underhome(&(line[1]))) {
+					line[0] = '~';
+					break;
+				}
 #endif
+/*FALLTHRU*/
 			case 'w':
+#ifdef	FD
 				if (!physical_path || !Xgetwd(line))
 					cp = fullpath;
+#else
+				if (!Xgetwd(line)) cp = vnullstr;
+#endif
 				break;
 			case 'W':
+#ifdef	FD
 				if (!physical_path || !Xgetwd(line))
 					tmp = fullpath;
+#else
+				if (!Xgetwd(line)) tmp = vnullstr;
+#endif
 				else tmp = line;
-#ifdef	_USEDOSPATH
+#ifdef	DEP_DOSPATH
 				if (_dospath(tmp)) tmp += 2;
 #endif
 				cp = strrdelim(tmp, 0);
 				if (cp && (cp != tmp || cp[1])) cp++;
 				else cp = tmp;
 				break;
-			case '~':
-				if (underhome(&(line[1]))) line[0] = '~';
-				else if (!physical_path || !Xgetwd(line))
-					cp = fullpath;
-				break;
 			case 'e':
 				*line = '\033';
 				line[1] = '\0';
@@ -681,7 +802,7 @@ char **bufp, *prompt;
 				unprint = 0;
 				break;
 			default:
-				tmp = sscanf2(&(prompt[i]), "%3Co", line);
+				tmp = sscanf2(&(prompt[i]), "%<3Co", line);
 				if (tmp) i = (tmp - prompt) - 1;
 				else *line = prompt[i];
 				line[1] = '\0';
@@ -718,19 +839,20 @@ char **bufp, *prompt;
 		}
 	}
 	(*bufp)[j] = '\0';
-	free(prompt);
+	free2(new);
+
 	return(len);
 }
 
-#ifndef	_NOARCHIVE
+#if	defined (FD) && !defined (_NOARCHIVE)
 char *getext(ext, flagsp)
-char *ext;
+CONST char *ext;
 u_char *flagsp;
 {
 	char *tmp;
 
 	*flagsp = 0;
-# if	FD >= 2
+# ifndef	OLDPARSE
 	if (*ext == '/') {
 		ext++;
 		*flagsp |= LF_IGNORECASE;
@@ -741,16 +863,17 @@ u_char *flagsp;
 	else {
 		tmp = malloc2(strlen(ext) + 2);
 		*tmp = '*';
-		strcpy(&(tmp[1]), ext);
+		strcpy2(&(tmp[1]), ext);
 	}
+
 	return(tmp);
 }
 
 /*ARGSUSED*/
 int extcmp(ext1, flags1, ext2, flags2, strict)
-char *ext1;
+CONST char *ext1;
 int flags1;
-char *ext2;
+CONST char *ext2;
 int flags2, strict;
 {
 	if (*ext1 == '*') ext1++;
@@ -760,17 +883,20 @@ int flags2, strict;
 	if ((flags1 & LF_IGNORECASE) || (flags2 & LF_IGNORECASE))
 		return(strcasecmp2(ext1, ext2));
 # endif
+
 	return(strpathcmp(ext1, ext2));
 }
-#endif	/* !_NOARCHIVE */
+#endif	/* FD && !_NOARCHIVE */
 
+#ifdef	FD
 int getkeycode(cp, identonly)
-char *cp;
+CONST char *cp;
 int identonly;
 {
 	char *tmp;
 	int i, ch;
 
+	if (!cp || !*cp) return(-1);
 	ch = (*(cp++) & 0xff);
 	if (!*cp) {
 		if (identonly) return(-1);
@@ -797,12 +923,12 @@ int identonly;
 			if (identonly) return(-1);
 			ch = (isalpha2(*cp)) ? mkmetakey(*(cp++)) : -1;
 			break;
-#ifdef	CODEEUC
+# ifdef	CODEEUC
 		case C_EKANA:
 			if (identonly) return(-1);
 			ch = (iskana2(*cp)) ? mkekana(*(cp++)) : -1;
 			break;
-#endif
+# endif
 		case 'F':
 			if ((i = atoi2(cp)) >= 1 && i <= 20) return(K_F(i));
 /*FALLTHRU*/
@@ -815,14 +941,23 @@ int identonly;
 			break;
 	}
 	if (*cp) ch = -1;
+
 	return(ch);
 }
 
-char *getkeysym(c, tenkey)
+CONST char *getkeysym(c, tenkey)
 int c, tenkey;
 {
-	static char buf[5];
-	int i, j;
+	static char buf[4 + 1];
+	int i;
+
+	for (i = 0; i < KEYIDENTSIZ; i++)
+		if ((u_short)(c) == keyidentlist[i].no) break;
+	if (i < KEYIDENTSIZ) {
+		if (c > (int)MAXUTYPE(u_char)
+		|| tenkey || c == ' ' || iscntrl2(c))
+			return(keyidentlist[i].str);
+	}
 
 	i = 0;
 	if (c >= K_F(1) && c <= K_F(20))
@@ -831,38 +966,35 @@ int c, tenkey;
 		buf[i++] = '@';
 		buf[i++] = c & 0x7f;
 	}
-#ifdef	CODEEUC
+# ifdef	CODEEUC
 	else if (isekana2(c)) {
 		buf[i++] = (char)C_EKANA;
 		buf[i++] = c & 0xff;
 	}
-#endif
+# else
+	else if (iskana2(c)) buf[i++] = c;
+# endif
 	else if (c > (int)MAXUTYPE(u_char)) {
-		for (j = 0; j < KEYIDENTSIZ; j++)
-			if ((u_short)(c) == keyidentlist[j].no) break;
-		if (j < KEYIDENTSIZ) {
-			if (tenkey || c == ' ' || iscntrl2(c)
-			|| keyidentlist[j].no >= K_MIN)
-				return(keyidentlist[j].str);
-		}
-
 		buf[i++] = '?';
 		buf[i++] = '?';
 	}
-#ifndef	CODEEUC
-	else if (iskana2(c)) buf[i++] = c;
-#endif
-	else if (iscntrl2(c))
-		i = snprintf2(buf, sizeof(buf), "^%c", (c + '@') & 0x7f);
+	else if (iscntrl2(c)) {
+		for (i = 0; escapechar[i]; i++)
+			if (c == escapevalue[i]) break;
+		if (escapechar[i])
+			i = snprintf2(buf, sizeof(buf), "\\%c", escapechar[i]);
+		else i = snprintf2(buf, sizeof(buf), "^%c", (c + '@') & 0x7f);
+	}
 	else if (ismsb(c)) i = snprintf2(buf, sizeof(buf), "\\%03o", c);
 	else buf[i++] = c;
-
 	buf[i] = '\0';
+
 	return(buf);
 }
+#endif	/* FD */
 
 char *decodestr(s, lenp, evalhat)
-char *s;
+CONST char *s;
 u_char *lenp;
 int evalhat;
 {
@@ -873,7 +1005,7 @@ int evalhat;
 	for (i = j = 0; s[i]; i++, j++) {
 		if (s[i] == '\\') {
 			i++;
-			if ((tmp = sscanf2(&(s[i]), "%3Co", &(cp[j])))) {
+			if ((tmp = sscanf2(&(s[i]), "%<3Co", &(cp[j])))) {
 				i = (tmp - s) - 1;
 				continue;
 			}
@@ -889,17 +1021,18 @@ int evalhat;
 		else cp[j] = s[i];
 	}
 
-	s = malloc2(j + 1);
-	memcpy(s, cp, j);
-	s[j] = '\0';
-	free(cp);
+	tmp = malloc2(j + 1);
+	memcpy(tmp, cp, j);
+	tmp[j] = '\0';
+	free2(cp);
 	if (lenp) *lenp = j;
-	return(s);
+
+	return(tmp);
 }
 
-#ifndef	_NOKEYMAP
+#if	defined (FD) && !defined (_NOKEYMAP)
 char *encodestr(s, len)
-char *s;
+CONST char *s;
 int len;
 {
 	char *cp;
@@ -908,10 +1041,8 @@ int len;
 	cp = malloc2(len * 4 + 1);
 	j = 0;
 	if (s) for (i = 0; i < len; i++) {
-		if (iskanji1(s, i)) cp[j++] = s[i++];
-# ifdef	CODEEUC
-		else if (isekana(s, i)) cp[j++] = s[i++];
-# else
+		if (iswchar(s, i)) cp[j++] = s[i++];
+# ifndef	CODEEUC
 		else if (isskana(s, i)) /*EMPTY*/;
 # endif
 		else if (iscntrl2(s[i]) || ismsb(s[i])) {
@@ -921,16 +1052,22 @@ int len;
 				cp[j++] = '\\';
 				cp[j++] = escapechar[n];
 			}
-			else if (iscntrl2(s[i]))
-				j = snprintf2(&(cp[j]), len * 4 + 1 - j,
+			else if (iscntrl2(s[i])) {
+				snprintf2(&(cp[j]), len * 4 + 1 - j,
 					"^%c", (s[i] + '@') & 0x7f);
-			else j += snprintf2(&(cp[j]), len * 4 + 1 - j,
-				"\\%03o", s[i] & 0xff);
+				j += strlen(&(cp[j]));
+			}
+			else {
+				snprintf2(&(cp[j]), len * 4 + 1 - j,
+					"\\%03o", s[i] & 0xff);
+				j += strlen(&(cp[j]));
+			}
 			continue;
 		}
 		cp[j++] = s[i];
 	}
 	cp[j] = '\0';
+
 	return(cp);
 }
-#endif	/* !_NOKEYMAP */
+#endif	/* FD && !_NOKEYMAP */
