@@ -4,100 +4,48 @@
  *	XSH command for POSIX
  */
 
-#include <fcntl.h>
 #ifdef	FD
 #include "fd.h"
-#else	/* !FD */
-#include <stdio.h>
-#include <string.h>
-#include <errno.h>
-#include "machine.h"
+#else
+#include "headers.h"
+#include "depend.h"
 #include "printf.h"
 #include "kctype.h"
-#include "pathname.h"
-
-#ifndef	NOUNISTDH
-#include <unistd.h>
+#include "string.h"
+#include "malloc.h"
 #endif
 
-#ifndef	NOSTDLIBH
-#include <stdlib.h>
-#endif
-#endif	/* !FD */
-
-#if	MSDOS
-#include <io.h>
+#include "sysemu.h"
 #include "unixemu.h"
-#else	/* !MSDOS */
-#include <sys/param.h>
-#include <sys/ioctl.h>
-#endif	/* !MSDOS */
 
+#if	!MSDOS
+#include "time.h"
+#endif
+#ifdef	DEP_ORIGSHELL
 #include "system.h"
-
-#if	!defined (MINIMUMSHELL) \
-&& (!defined (FD) || (FD >= 2 && !defined (_NOORIGSHELL)))
-
+#endif
+#if	defined (DEP_ORIGSHELL) && !defined (MINIMUMSHELL)
+#include "posixsh.h"
+#endif
 #ifdef	FD
 #include "term.h"
-extern int Xstat __P_((char *, struct stat *));
-extern int Xlstat __P_((char *, struct stat *));
-extern int Xaccess __P_((char *, int));
-#else	/* !FD */
-extern int ttyio;
-extern FILE *ttyout;
-# if	MSDOS
-extern int Xstat __P_((char *, struct stat *));
-# define	Xlstat		Xstat
-# else
-# define	Xstat		stat
-# define	Xlstat		lstat
-# endif
-#define	Xaccess(p, m)	(access(p, m) ? -1 : 0)
-#endif	/* !FD */
-
-#ifndef	EPERM
-#define	EPERM		EACCES
+#endif
+#ifdef	DEP_PTY
+#include "termemu.h"
 #endif
 
-extern VOID error __P_((char *));
-extern char *malloc2 __P_((ALLOC_T));
-extern char *realloc2 __P_((VOID_P, ALLOC_T));
-extern char *strdup2 __P_((char *));
-extern char *strndup2 __P_((char *, int));
-extern char *strchr2 __P_((char *, int));
-extern char *strncpy2 __P_((char *, char *, int));
-extern int setenv2 __P_((char *, char *, int));
-extern time_t time2 __P_((VOID_A));
-
-#ifndef	NOJOB
-static char *NEAR headstree __P_((syntaxtree *));
-#endif
-#ifndef	NOALIAS
-static shaliastable *NEAR duplalias __P_((shaliastable *));
-static int cmpalias __P_((CONST VOID_P, CONST VOID_P));
-#endif
-static int NEAR _evalexpression __P_((char *, int, long *));
-static int NEAR evalexpression __P_((char *, int, long *, int));
-#if	!MSDOS
-static VOID NEAR addmailpath __P_((char *, char *, time_t));
-#endif
-static int NEAR testsub1 __P_((int, char *, int *));
-static int NEAR testsub2 __P_((int, char *[], int *));
-static int NEAR testsub3 __P_((int, char *[], int *, int));
-
-#define	ALIASDELIMIT	"();&|<>"
-#define	BUFUNIT		32
-#define	getconstvar(s)	(getshellvar(s, sizeof(s) - 1))
-#define	ER_COMNOFOUND	1
-#define	ER_NOTFOUND	2
-#define	ER_NOTIDENT	4
-#define	ER_BADOPTIONS	13
-#define	ER_MISSARG	15
-#define	ER_NOTALIAS	19
-#define	ER_NOSUCHJOB	20
-#define	ER_NUMOUTRANGE	21
-#define	ER_UNKNOWNSIG	22
+#define	ALIASDELIMIT		"();&|<>"
+#define	getconstvar(s)		(getshellvar(s, strsize(s)))
+#define	ER_COMNOFOUND		1
+#define	ER_NOTFOUND		2
+#define	ER_NOTIDENT		4
+#define	ER_BADOPTIONS		13
+#define	ER_NUMOUTRANGE		18
+#define	ER_NOTALIAS		20
+#define	ER_MISSARG		21
+#define	ER_NOSUCHJOB		22
+#define	ER_TERMINATED		23
+#define	ER_UNKNOWNSIG		26
 
 #if	!MSDOS
 typedef struct _mailpath_t {
@@ -107,40 +55,34 @@ typedef struct _mailpath_t {
 } mailpath_t;
 #endif
 
+#if	defined (DEP_ORIGSHELL) && !defined (MINIMUMSHELL)
+
+extern int setenv2 __P_((CONST char *, CONST char *, int));
+#ifdef	DEP_PTY
+extern VOID sendparent __P_((int, ...));
+#endif
+
+#ifndef	FD
+extern int ttyio;
+extern XFILE *ttyout;
+#endif
+
 #ifndef	NOJOB
-int gettermio __P_((p_id_t));
-VOID dispjob __P_((int, FILE *));
-int searchjob __P_((p_id_t, int *));
-int getjob __P_((char *));
-int stackjob __P_((p_id_t, int, syntaxtree *));
-int stoppedjob __P_((p_id_t));
-VOID killjob __P_((VOID_A));
-VOID checkjob __P_((int));
-#endif	/* !NOJOB */
-char *evalposixsubst __P_((char *, int *));
+static CONST char *NEAR headstree __P_((syntaxtree *));
+#endif
+#ifndef	NOALIAS
+static int NEAR searchalias __P_((CONST char *, int));
+static shaliastable *NEAR duplalias __P_((shaliastable *));
+static int cmpalias __P_((CONST VOID_P, CONST VOID_P));
+#endif
+static int NEAR _evalexpression __P_((CONST char *, int, long *));
+static int NEAR evalexpression __P_((CONST char *, int, long *, int));
 #if	!MSDOS
-VOID replacemailpath __P_((char *, int));
-VOID checkmail __P_((int));
-#endif
-#ifndef	NOALIAS
-VOID freealias __P_((shaliastable *));
-int checkalias __P_((syntaxtree *, char *, int, int));
-#endif
-#ifndef	NOJOB
-int posixjobs __P_((syntaxtree *));
-int posixfg __P_((syntaxtree *));
-int posixbg __P_((syntaxtree *));
-#endif
-#ifndef	NOALIAS
-int posixalias __P_((syntaxtree *));
-int posixunalias __P_((syntaxtree *));
-#endif
-int posixkill __P_((syntaxtree *));
-int posixtest __P_((syntaxtree *));
-#ifndef	NOPOSIXUTIL
-int posixcommand __P_((syntaxtree *));
-int posixgetopts __P_((syntaxtree *));
+static VOID NEAR addmailpath __P_((CONST char *, char *, time_t));
 #endif
+static int NEAR testsub1 __P_((int, CONST char *, int *));
+static int NEAR testsub2 __P_((int, char *CONST *, int *));
+static int NEAR testsub3 __P_((int, char *CONST *, int *, int));
 
 #ifndef	NOJOB
 jobtable *joblist = NULL;
@@ -163,15 +105,16 @@ static int maxmailpath = 0;
 
 
 #ifndef	NOJOB
-int gettermio(pgrp)
+int gettermio(pgrp, job)
 p_id_t pgrp;
+int job;
 {
 	int ret;
 	sigmask_t mask, omask;
 
 	if (ttypgrp < (p_id_t)0 || pgrp < (p_id_t)0 || pgrp == ttypgrp)
 		return(0);
-	else if (!jobok) {
+	else if (!job) {
 		ttypgrp = pgrp;
 		return(0);
 	}
@@ -193,14 +136,15 @@ p_id_t pgrp;
 
 #ifdef	JOBVERBOSE
 	fprintf2(ttyout, "gettermio: %id: %id -> %id", mypid, ttypgrp, pgrp);
-	fputnl(ttyout);
+	VOID_C fputnl(ttyout);
 #endif	/* JOBVERBOSE */
 	if ((ret = settcpgrp(ttyio, pgrp)) >= 0) ttypgrp = pgrp;
 	Xsigsetmask(omask);
+
 	return(ret);
 }
 
-static char *NEAR headstree(trp)
+static CONST char *NEAR headstree(trp)
 syntaxtree *trp;
 {
 	int id;
@@ -216,12 +160,13 @@ syntaxtree *trp;
 	if (!trp || !(trp -> comm)) return(NULL);
 	id = (trp -> comm) -> id;
 	if (id == SM_CHILD || id == SM_STATEMENT) return(NULL);
+
 	return(statementlist[id - 1].ident);
 }
 
 VOID dispjob(n, fp)
 int n;
-FILE *fp;
+XFILE *fp;
 {
 	int i, sig;
 
@@ -244,7 +189,7 @@ FILE *fp;
 			joblist[n].trp -> type = OP_NONE;
 		printstree(joblist[n].trp, -1, fp);
 	}
-	fputnl(fp);
+	VOID_C fputnl(fp);
 }
 
 int searchjob(pid, np)
@@ -261,13 +206,14 @@ int *np;
 			return(i);
 		}
 	}
+
 	return(-1);
 }
 
 int getjob(s)
-char *s;
+CONST char *s;
 {
-	char *cp;
+	CONST char *cp;
 	int i, j;
 
 	if (!jobok) return(-1);
@@ -291,7 +237,7 @@ char *s;
 	if (i < 0 || i >= maxjobs || !joblist || !(joblist[i].pids))
 		return(-1);
 	j = joblist[i].npipe;
-	if (joblist[i].stats[j] < 0 || joblist[i].stats[j] >= 128) return(-1);
+	if (joblist[i].stats[j] < 0 || joblist[i].stats[j] >= 128) return(-2);
 
 	return(i);
 }
@@ -361,11 +307,7 @@ syntaxtree *trp;
 	joblist[i].pids[j] = pid;
 	joblist[i].stats[j] = sig;
 	if (!j && !(joblist[i].trp) && trp) {
-#ifdef	MINIMUMSHELL
-		joblist[i].trp = duplstree(trp, NULL);
-#else
 		joblist[i].trp = duplstree(trp, NULL, 0L);
-#endif
 		if (!isoppipe(joblist[i].trp) && joblist[i].trp -> next) {
 			freestree(joblist[i].trp -> next);
 			joblist[i].trp -> next = NULL;
@@ -376,8 +318,9 @@ syntaxtree *trp;
 	fprintf2(ttyout, "stackjob: %id: %id, %d:", mypid, pid, i);
 	for (j = 0; j <= joblist[i].npipe; j++)
 		fprintf2(ttyout, "%id ", joblist[i].pids[j]);
-	fputnl(ttyout);
+	VOID_C fputnl(ttyout);
 #endif	/* JOBVERBOSE */
+
 	return(i);
 }
 
@@ -394,16 +337,17 @@ p_id_t pid;
 			else if (!sig) return(0);
 		}
 	}
+
 	return(-1);
 }
 
-VOID checkjob(verbose)
-int verbose;
+VOID checkjob(fp)
+XFILE *fp;
 {
 	int i, j;
 
 	while (waitjob(-1, NULL, WNOHANG | WUNTRACED) > 0);
-	if (verbose) for (i = 0; i < maxjobs; i++) {
+	if (fp) for (i = 0; i < maxjobs; i++) {
 		if (!(joblist[i].pids)) continue;
 		j = joblist[i].npipe;
 		if (joblist[i].stats[j] >= 0 && joblist[i].stats[j] < 128)
@@ -412,16 +356,16 @@ int verbose;
 		if (joblist[i].trp
 		&& (isopbg(joblist[i].trp) || isopnown(joblist[i].trp)))
 			joblist[i].trp -> type = OP_NONE;
-		if (jobok && interactive && !nottyout) dispjob(i, stderr);
-		free(joblist[i].pids);
-		free(joblist[i].stats);
+		if (jobok && interactive && !nottyout) dispjob(i, fp);
+		free2(joblist[i].pids);
+		free2(joblist[i].stats);
 		if (joblist[i].trp) {
 			freestree(joblist[i].trp);
-			free(joblist[i].trp);
+			free2(joblist[i].trp);
 		}
-		if (joblist[i].tty) free(joblist[i].tty);
+		free2(joblist[i].tty);
 # ifdef	USESGTTY
-		if (joblist[i].ttyflag) free(joblist[i].ttyflag);
+		free2(joblist[i].ttyflag);
 # endif
 		joblist[i].pids = NULL;
 	}
@@ -442,6 +386,76 @@ VOID killjob(VOID_A)
 #endif	/* !NOJOB */
 
 #ifndef	NOALIAS
+static int NEAR searchalias(ident, len)
+CONST char *ident;
+int len;
+{
+	int i;
+
+	if (len < 0) len = strlen(ident);
+	for (i = 0; shellalias[i].ident; i++)
+		if (!strncommcmp(ident, shellalias[i].ident, len)
+		&& !(shellalias[i].ident[len]))
+			break;
+
+	return(i);
+}
+
+int addalias(ident, comm)
+char *ident, *comm;
+{
+	int i;
+
+	i = searchalias(ident, -1);
+	if (shellalias[i].ident) {
+		free2(shellalias[i].ident);
+		free2(shellalias[i].comm);
+	}
+	else {
+		shellalias = (shaliastable *)realloc2(shellalias,
+			(i + 2) * sizeof(shaliastable));
+		shellalias[i + 1].ident = NULL;
+	}
+
+	shellalias[i].ident = ident;
+	shellalias[i].comm = comm;
+# ifdef	DEP_PTY
+	sendparent(TE_ADDALIAS, ident, comm);
+# endif
+
+	return(0);
+}
+
+int deletealias(ident)
+CONST char *ident;
+{
+	reg_t *re;
+	int i, n, max;
+
+	n = 0;
+	re = regexp_init(ident, -1);
+	for (max = 0; shellalias[max].ident; max++) /*EMPTY*/;
+	for (i = 0; i < max; i++) {
+		if (re) {
+			if (!regexp_exec(re, shellalias[i].ident, 0)) continue;
+		}
+		else if (strcommcmp(ident, shellalias[i].ident)) continue;
+		free2(shellalias[i].ident);
+		free2(shellalias[i].comm);
+		memmove((char *)&(shellalias[i]), (char *)&(shellalias[i + 1]),
+			(max-- - i) * sizeof(shaliastable));
+		i--;
+		n++;
+	}
+	regexp_free(re);
+	if (!n) return(-1);
+# ifdef	DEP_PTY
+	sendparent(TE_DELETEALIAS, ident);
+# endif
+
+	return(0);
+}
+
 static shaliastable *NEAR duplalias(alias)
 shaliastable *alias;
 {
@@ -449,13 +463,14 @@ shaliastable *alias;
 	int i, n;
 
 	if (!alias) n = 0;
-	else for (n = 0; alias[n].ident; n++);
+	else for (n = 0; alias[n].ident; n++) /*EMPTY*/;
 	dupl = (shaliastable *)malloc2((n + 1) * sizeof(shaliastable));
 	for (i = 0; i < n; i++) {
 		dupl[i].ident = strdup2(alias[i].ident);
 		dupl[i].comm = strdup2(alias[i].comm);
 	}
 	dupl[i].ident = NULL;
+
 	return(dupl);
 }
 
@@ -466,10 +481,10 @@ shaliastable *alias;
 
 	if (alias) {
 		for (i = 0; alias[i].ident; i++) {
-			free(alias[i].ident);
-			free(alias[i].comm);
+			free2(alias[i].ident);
+			free2(alias[i].comm);
 		}
-		free(alias);
+		free2(alias);
 	}
 }
 
@@ -481,6 +496,7 @@ CONST VOID_P vp2;
 
 	ap1 = (shaliastable *)vp1;
 	ap2 = (shaliastable *)vp2;
+
 	return(strpathcmp2(ap1 -> ident, ap2 -> ident));
 }
 
@@ -497,30 +513,29 @@ int len, delim;
 	&& !(statementlist[i].type & STT_NEEDLIST)))
 		return(-1);
 
-	if ((!strchr(IFS_SET, delim) && !strchr(ALIASDELIMIT, delim)))
+	if (delim
+	&& (!strchr2(IFS_SET, delim) && !strchr2(ALIASDELIMIT, delim)))
+		return(-1);
+	i = searchalias(ident, len);
+	if (!(shellalias[i].ident) || (shellalias[i].flags & AL_USED))
 		return(-1);
-	for (i = 0; shellalias[i].ident; i++)
-		if (!(shellalias[i].flags & AL_USED)
-		&& !strncommcmp(ident, shellalias[i].ident, len)
-		&& !(shellalias[i].ident[len]))
-			break;
-	if (!(shellalias[i].ident)) return(-1);
 
 	*ident = '\0';
 	shellalias[i].flags |= AL_USED;
+
 	return(i);
 }
 #endif	/* NOALIAS */
 
 static int NEAR _evalexpression(s, ptr, resultp)
-char *s;
+CONST char *s;
 int ptr;
 long *resultp;
 {
 	long n;
 	int i, c, base;
 
-	while (s[ptr] && strchr(IFS_SET, s[ptr])) ptr++;
+	while (s[ptr] && strchr2(IFS_SET, s[ptr])) ptr++;
 	if (!s[ptr]) return(-1);
 
 	if (s[ptr] == '+') {
@@ -545,7 +560,7 @@ long *resultp;
 	else if (s[ptr] == '(') {
 		if ((ptr = evalexpression(s, ptr + 1, resultp, 9)) < 0)
 			return(-1);
-		while (strchr(IFS_SET, s[ptr])) ptr++;
+		while (s[ptr] && strchr2(IFS_SET, s[ptr])) ptr++;
 		if (s[ptr++] != ')') return(-1);
 	}
 	else if (isdigit2(s[ptr])) {
@@ -577,12 +592,13 @@ long *resultp;
 	}
 	else return(-1);
 
-	while (s[ptr] && strchr(IFS_SET, s[ptr])) ptr++;
+	while (s[ptr] && strchr2(IFS_SET, s[ptr])) ptr++;
+
 	return(ptr);
 }
 
 static int NEAR evalexpression(s, ptr, resultp, lvl)
-char *s;
+CONST char *s;
 int ptr;
 long *resultp;
 int lvl;
@@ -726,10 +742,14 @@ int lvl;
 }
 
 char *evalposixsubst(s, ptrp)
-char *s;
+CONST char *s;
 int *ptrp;
 {
-	char *cp;
+#ifndef	BASHBUG
+	syntaxtree *trp, *stree;
+	redirectlist red;
+#endif
+	char *cp, *new;
 	long n;
 	int i, len;
 
@@ -738,23 +758,20 @@ int *ptrp;
 
 	s += i;
 	len = *ptrp - i * 2;
-	s = strndup2(s, len);
+	s = new = strndup2(s, len);
 	if (i <= 1) {
 #ifndef	BASHBUG
 	/* bash cannot include 'case' statement within $() */
-		syntaxtree *trp, *stree;
-		redirectlist red;
-
 		red.fd = red.type = red.new = 0;
-		red.filename = "";
+		red.filename = vnullstr;
 		i = len = 0;
 		stree = newstree(NULL);
-		trp = startvar(stree, &red, "", &i, &len, 0);
+		trp = startvar(stree, &red, nullstr, &i, &len, 0);
 		trp -> cont = CN_COMM;
 		trp = analyze(s, trp, -1);
 		if (trp && (trp -> cont & CN_SBST) != CN_CASE) trp = NULL;
 		freestree(stree);
-		free(stree);
+		free2(stree);
 		if (trp) cp = NULL;
 		else
 #endif	/* !BASHBUG */
@@ -762,36 +779,38 @@ int *ptrp;
 	}
 	else {
 		n = 0L;
-		if (!(cp = evalvararg(s, '\'', EA_BACKQ, 0))) *ptrp = -1;
+		cp = evalvararg(new,
+			EA_STRIPESCAPE
+			| EA_NOEVALQ | EA_NOEVALDQ | EA_BACKQ, 0);
+		if (!cp) *ptrp = -1;
 		else {
-			for (i = 0; cp[i] && strchr(IFS_SET, cp[i]); i++);
+			for (i = 0; cp[i]; i++)
+				if (!strchr2(IFS_SET, cp[i])) break;
 			if (cp[i]) {
 				i = evalexpression(cp, i, &n, 9);
 				if (i < 0 || cp[i]) *ptrp = -1;
 			}
-			free(cp);
+			free2(cp);
 		}
 
-		if (*ptrp < 0) cp = NULL;
-		else if (asprintf2(&cp, "%ld", n) < 0) error("malloc()");
+		cp = (*ptrp < 0) ? NULL : asprintf3("%ld", n);
 	}
-	free(s);
+	free2(new);
 
 	return(cp);
 }
 
 #if	!MSDOS
 static VOID NEAR addmailpath(path, msg, mtime)
-char *path, *msg;
+CONST char *path;
+char *msg;
 time_t mtime;
 {
 	int i;
 
 	for (i = 0; i < maxmailpath; i++)
 		if (!strpathcmp(path, mailpathlist[i].path)) break;
-	if (i < maxmailpath) {
-		if (mailpathlist[i].msg) free(mailpathlist[i].msg);
-	}
+	if (i < maxmailpath) free2(mailpathlist[i].msg);
 	else {
 		mailpathlist = (mailpath_t *)realloc2(mailpathlist,
 			(i + 1) * sizeof(mailpath_t));
@@ -803,12 +822,13 @@ time_t mtime;
 }
 
 VOID replacemailpath(s, multi)
-char *s;
+CONST char *s;
 int multi;
 {
 	mailpath_t *mailpath;
 	time_t mtime;
-	char *cp, *msg, path[MAXPATHLEN];
+	CONST char *cp;
+	char *msg, path[MAXPATHLEN];
 	int i, max, len;
 
 	mailpath = mailpathlist;
@@ -823,12 +843,12 @@ int multi;
 		addmailpath(s, NULL, mtime);
 	}
 	else while (*s) {
-		if ((cp = strchr(s, ':'))) len = (cp++) - s;
+		if ((cp = strchr2(s, ':'))) len = (cp++) - s;
 		else {
 			len = strlen(s);
-			cp = s + len;
+			cp = &(s[len]);
 		}
-		if ((msg = strchr(s, '%'))) {
+		if ((msg = strchr2(s, '%'))) {
 			if (msg > s + len) msg = NULL;
 			else {
 				i = s + len - (msg + 1);
@@ -845,10 +865,10 @@ int multi;
 	}
 
 	for (i = 0; i < max; i++) {
-		free(mailpath[i].path);
-		if (mailpath[i].msg) free(mailpath[i].msg);
+		free2(mailpath[i].path);
+		free2(mailpath[i].msg);
 	}
-	if (mailpath) free(mailpath);
+	free2(mailpath);
 }
 
 VOID checkmail(reset)
@@ -860,10 +880,10 @@ int reset;
 
 	if (reset) {
 		for (i = 0; i < maxmailpath; i++) {
-			free(mailpathlist[i].path);
-			if (mailpathlist[i].msg) free(mailpathlist[i].msg);
+			free2(mailpathlist[i].path);
+			free2(mailpathlist[i].msg);
 		}
-		free(mailpathlist);
+		free2(mailpathlist);
 		mailpathlist = NULL;
 		maxmailpath = 0;
 		return;
@@ -889,26 +909,27 @@ syntaxtree *trp;
 	int i, j;
 
 	if (mypid != orgpgrp) return(RET_SUCCESS);
-	checkjob(0);
+	checkjob(NULL);
 	for (i = 0; i < maxjobs; i++) {
 		if (!(joblist[i].pids)) continue;
 		j = joblist[i].npipe;
 
 		if (joblist[i].stats[j] >= 0 && joblist[i].stats[j] < 128)
-			dispjob(i, stdout);
+			dispjob(i, Xstdout);
 		else {
 			if (isopbg(joblist[i].trp) || isopnown(joblist[i].trp))
 				joblist[i].trp -> type = OP_NONE;
-			dispjob(i, stdout);
-			free(joblist[i].pids);
-			free(joblist[i].stats);
+			dispjob(i, Xstdout);
+			free2(joblist[i].pids);
+			free2(joblist[i].stats);
 			if (joblist[i].trp) {
 				freestree(joblist[i].trp);
-				free(joblist[i].trp);
+				free2(joblist[i].trp);
 			}
 			joblist[i].pids = NULL;
 		}
 	}
+
 	return(RET_SUCCESS);
 }
 
@@ -921,9 +942,11 @@ syntaxtree *trp;
 	int i, j, n, ret;
 
 	s = ((trp -> comm) -> argc > 1) ? (trp -> comm) -> argv[1] : NULL;
-	checkjob(0);
+	checkjob(NULL);
 	if ((i = getjob(s)) < 0) {
-		execerror((trp -> comm) -> argv[1], ER_NOSUCHJOB, 0);
+		execerror((trp -> comm) -> argv,
+			(trp -> comm) -> argv[1],
+			(i < -1) ? ER_TERMINATED : ER_NOSUCHJOB, 2);
 		return(RET_FAIL);
 	}
 	n = joblist[i].npipe;
@@ -932,13 +955,13 @@ syntaxtree *trp;
 		prepareexit(-1);
 		Xexit(RET_FATALERR);
 	}
-	fprintf2(stderr, "[%d] %id", i + 1, joblist[i].pids[n]);
-	fputnl(stderr);
+	fprintf2(Xstderr, "[%d] %id", i + 1, joblist[i].pids[n]);
+	VOID_C fputnl(Xstderr);
 	if (joblist[i].tty) tioctl(ttyio, REQSETP, joblist[i].tty);
 # ifdef	USESGTTY
 	if (joblist[i].ttyflag) ioctl(ttyio, TIOCLSET, joblist[i].ttyflag);
 # endif
-	gettermio(joblist[i].pids[0]);
+	gettermio(joblist[i].pids[0], jobok);
 	if ((j = joblist[i].stats[n]) > 0 && j < 128) {
 		Xkillpg(joblist[i].pids[0], SIGCONT);
 		for (j = 0; j <= n; j++) joblist[i].stats[j] = 0;
@@ -946,12 +969,13 @@ syntaxtree *trp;
 	if (isopbg(joblist[i].trp) || isopnown(joblist[i].trp))
 		joblist[i].trp -> type = OP_NONE;
 	ret = waitchild(joblist[i].pids[n], NULL);
-	gettermio(orgpgrp);
+	gettermio(orgpgrp, jobok);
 	if (tioctl(ttyio, REQSETP, &tty) < 0) {
 		doperror((trp -> comm) -> argv[0], "fatal error");
 		prepareexit(-1);
 		Xexit(RET_FATALERR);
 	}
+
 	return(ret);
 }
 
@@ -963,9 +987,11 @@ syntaxtree *trp;
 	int i, j, n;
 
 	s = ((trp -> comm) -> argc > 1) ? (trp -> comm) -> argv[1] : NULL;
-	checkjob(0);
+	checkjob(NULL);
 	if ((i = getjob(s)) < 0) {
-		execerror((trp -> comm) -> argv[1], ER_NOSUCHJOB, 0);
+		execerror((trp -> comm) -> argv,
+			(trp -> comm) -> argv[1],
+			(i < -1) ? ER_TERMINATED : ER_NOSUCHJOB, 2);
 		return(RET_FAIL);
 	}
 	n = joblist[i].npipe;
@@ -975,9 +1001,10 @@ syntaxtree *trp;
 	}
 	if (!isopbg(joblist[i].trp) || !isopnown(joblist[i].trp))
 		joblist[i].trp -> type = OP_BG;
-	if (interactive && !nottyout) dispjob(i, stderr);
+	if (interactive && !nottyout) dispjob(i, Xstderr);
 	prevjob = lastjob;
 	lastjob = i;
+
 	return(RET_SUCCESS);
 }
 #endif	/* !NOJOB */
@@ -993,12 +1020,12 @@ syntaxtree *trp;
 	argv = (trp -> comm) -> argv;
 	if ((trp -> comm) -> argc <= 1) {
 		alias = duplalias(shellalias);
-		for (i = 0; alias[i].ident; i++);
+		for (i = 0; alias[i].ident; i++) /*EMPTY*/;
 		if (i > 1) qsort(alias, i, sizeof(shaliastable), cmpalias);
 		for (i = 0; alias[i].ident; i++) {
-			fprintf2(stdout, "alias %k='%k'",
+			fprintf2(Xstdout, "alias %k='%k'",
 				alias[i].ident, alias[i].comm);
-			fputnl(stdout);
+			VOID_C fputnl(Xstdout);
 		}
 		freealias(alias);
 		return(RET_SUCCESS);
@@ -1011,77 +1038,48 @@ syntaxtree *trp;
 			set = 1;
 			break;
 		}
-		for (i = 0; shellalias[i].ident; i++)
-			if (!strncommcmp(shellalias[i].ident, argv[n], len)
-			&& !(shellalias[i].ident[len]))
-				break;
 
-		if (!set) {
+		if (set) VOID_C addalias(strndup2(argv[n], len),
+			strdup2(&(argv[n][len + 1])));
+		else {
+			i = searchalias(argv[n], len);
 			if (!(shellalias[i].ident)) {
 				if (interactive) {
-					fputs("alias: ", stderr);
-					execerror(argv[n], ER_NOTFOUND, 0);
+					Xfputs("alias: ", Xstderr);
+					execerror(argv,
+						argv[n], ER_NOTFOUND, 2);
 				}
 				ret = RET_FAIL;
 				ERRBREAK;
 			}
-			fprintf2(stdout, "alias %k='%k'",
+			fprintf2(Xstdout, "alias %k='%k'",
 				shellalias[i].ident, shellalias[i].comm);
-			fputnl(stdout);
-		}
-		else if (shellalias[i].ident) {
-			free(shellalias[i].comm);
-			shellalias[i].comm = strdup2(&(argv[n][++len]));
-		}
-		else {
-			shellalias = (shaliastable *)realloc2(shellalias,
-				(i + 2) * sizeof(shaliastable));
-			shellalias[i].ident = strndup2(argv[n], len);
-			shellalias[i].comm = strdup2(&(argv[n][++len]));
-			shellalias[++i].ident = NULL;
+			VOID_C fputnl(Xstdout);
 		}
 	}
+
 	return(ret);
 }
 
 int posixunalias(trp)
 syntaxtree *trp;
 {
-	reg_t *re;
 	char **argv;
-	int i, n, c, ret;
+	int n, ret;
 
 	argv = (trp -> comm) -> argv;
 	ret = RET_SUCCESS;
 	for (n = 1; n < (trp -> comm) -> argc; n++) {
-		c = 0;
-		re = regexp_init(argv[n], -1);
-		for (i = 0; shellalias[i].ident; i++) {
-			if (re) {
-				if (!regexp_exec(re, shellalias[i].ident, 0))
-					continue;
-			}
-			else if (strcommcmp(shellalias[i].ident, argv[n]))
-				continue;
-			c++;
-			free(shellalias[i].ident);
-			free(shellalias[i].comm);
-			for (; shellalias[i + 1].ident; i++) {
-				shellalias[i].ident = shellalias[i + 1].ident;
-				shellalias[i].comm = shellalias[i + 1].comm;
-			}
-			shellalias[i].ident = NULL;
-		}
-		if (re) regexp_free(re);
-		if (!c) {
+		if (deletealias(argv[n]) < 0) {
 			if (interactive) {
-				fputs("alias: ", stderr);
-				execerror(argv[n], ER_NOTALIAS, 0);
+				Xfputs("alias: ", Xstderr);
+				execerror(argv, argv[n], ER_NOTALIAS, 2);
 			}
 			ret = RET_FAIL;
 			ERRBREAK;
 		}
 	}
+
 	return(ret);
 }
 #endif	/* NOALIAS */
@@ -1094,9 +1092,9 @@ syntaxtree *trp;
 	int i, n, sig;
 
 	if ((trp -> comm) -> argc <= 1) {
-		fputs("usage: kill [ -sig ] pid ...\n", stderr);
-		fputs("for a list of signals: kill -l", stderr);
-		fputnl(stderr);
+		Xfputs("usage: kill [ -sig ] pid ...\n", Xstderr);
+		Xfputs("for a list of signals: kill -l", Xstderr);
+		VOID_C fputnl(Xstderr);
 		return(RET_SYNTAXERR);
 	}
 #ifdef	SIGTERM
@@ -1112,12 +1110,12 @@ syntaxtree *trp;
 				for (i = 0; signallist[i].sig >= 0; i++)
 					if (sig == signallist[i].sig) break;
 				if (signallist[i].sig < 0) continue;
-				fprintf2(stdout, "%s%c",
+				fprintf2(Xstdout, "%s%c",
 					signallist[i].ident,
 					(++n % 16) ? ' ' : '\n');
 			}
-			if (n % 16) fputnl(stdout);
-			fflush(stdout);
+			if (n % 16) VOID_C fputnl(Xstdout);
+			Xfflush(Xstdout);
 			return(RET_SUCCESS);
 		}
 		if ((sig = isnumeric(&(s[1]))) < 0) {
@@ -1125,27 +1123,31 @@ syntaxtree *trp;
 				if (!strcmp(&(s[1]), signallist[i].ident))
 					break;
 			if (signallist[i].sig < 0) {
-				execerror(&(s[1]), ER_UNKNOWNSIG, 0);
+				execerror((trp -> comm) -> argv,
+					&(s[1]), ER_UNKNOWNSIG, 1);
 				return(RET_FAIL);
 			}
 			sig = signallist[i].sig;
 		}
 		if (sig < 0 || sig >= NSIG) {
-			execerror(s, ER_NUMOUTRANGE, 0);
+			execerror((trp -> comm) -> argv, s, ER_NUMOUTRANGE, 1);
 			return(RET_FAIL);
 		}
 		i = 2;
 	}
 
 #ifndef	NOJOB
-	checkjob(0);
+	checkjob(NULL);
 #endif
 	for (; i < (trp -> comm) -> argc; i++) {
 		s = (trp -> comm) -> argv[i];
 #ifndef	NOJOB
 		if (*s == '%') {
 			if ((n = getjob(s)) < 0) {
-				execerror(s, ER_NOSUCHJOB, 0);
+				execerror((trp -> comm) -> argv,
+					s,
+					(n < -1)
+					? ER_TERMINATED : ER_NOSUCHJOB, 2);
 				return(RET_FAIL);
 			}
 			n = Xkillpg(joblist[n].pids[0], sig);
@@ -1153,16 +1155,13 @@ syntaxtree *trp;
 		else
 #endif	/* !NOJOB */
 		if ((pid = isnumeric(s)) < (p_id_t)0) {
-			fputs("usage: kill [ -sig ] pid ...\n", stderr);
-			fputs("for a list of signals: kill -l", stderr);
-			fputnl(stderr);
+			Xfputs("usage: kill [ -sig ] pid ...\n", Xstderr);
+			Xfputs("for a list of signals: kill -l", Xstderr);
+			VOID_C fputnl(Xstderr);
 			return(RET_SYNTAXERR);
 		}
 #if	MSDOS
-		else if (pid && pid != mypid) {
-			errno = EPERM;
-			n = -1;
-		}
+		else if (pid && pid != mypid) n = seterrno(EPERM);
 #endif
 		else n = kill(pid, sig);
 
@@ -1171,12 +1170,13 @@ syntaxtree *trp;
 			return(RET_FAIL);
 		}
 	}
+
 	return(RET_SUCCESS);
 }
 
 static int NEAR testsub1(c, s, ptrp)
 int c;
-char *s;
+CONST char *s;
 int *ptrp;
 {
 	struct stat st;
@@ -1197,8 +1197,14 @@ int *ptrp;
 				? RET_SUCCESS : RET_FAIL;
 			break;
 		case 'f':
+#if	defined (BASHSTYLE) || defined (STRICTPOSIX)
+	/* "test" on bash & POSIX regards -f as checking if regular file. */
+			if (s) ret = (*s && Xstat(s, &st) >= 0
+			&& (st.st_mode & S_IFMT) == S_IFREG)
+#else
 			if (s) ret = (*s && Xstat(s, &st) >= 0
 			&& (st.st_mode & S_IFMT) != S_IFDIR)
+#endif
 				? RET_SUCCESS : RET_FAIL;
 			break;
 		case 'd':
@@ -1286,15 +1292,16 @@ int *ptrp;
 			break;
 	}
 	if (ret >= 0) *ptrp += 2;
+
 	return(ret);
 }
 
 static int NEAR testsub2(argc, argv, ptrp)
 int argc;
-char *argv[];
+char *CONST *argv;
 int *ptrp;
 {
-	char *s, *a1, *a2;
+	CONST char *s, *a1, *a2;
 	int ret, v1, v2;
 
 	if (*ptrp >= argc) return(-1);
@@ -1316,6 +1323,11 @@ int *ptrp;
 
 		if (s[0] == '!' && s[1] == '=' && !s[2])
 			ret = (strcmp(a1, a2)) ? RET_SUCCESS : RET_FAIL;
+#ifdef	BASHSTYLE
+	/* bash's "test" allows "==" as same as "=" */
+		else if (s[0] == '=' && s[1] == '=' && !s[2])
+			ret = (!strcmp(a1, a2)) ? RET_SUCCESS : RET_FAIL;
+#endif
 		else if (s[0] == '=' && !s[1])
 			ret = (!strcmp(a1, a2)) ? RET_SUCCESS : RET_FAIL;
 		else if (s[0] == '-') {
@@ -1365,12 +1377,13 @@ int *ptrp;
 	}
 	ret = (argv[*ptrp][0]) ? RET_SUCCESS : RET_FAIL;
 	(*ptrp)++;
+
 	return(ret);
 }
 
 static int NEAR testsub3(argc, argv, ptrp, lvl)
 int argc;
-char *argv[];
+char *CONST *argv;
 int *ptrp, lvl;
 {
 	int ret1, ret2;
@@ -1396,13 +1409,14 @@ int *ptrp, lvl;
 		? (ret1 != RET_FAIL && ret2 != RET_FAIL)
 		: (ret1 != RET_FAIL || ret2 != RET_FAIL))
 			? RET_SUCCESS : RET_FAIL;
+
 	return(ret1);
 }
 
 int posixtest(trp)
 syntaxtree *trp;
 {
-	char **argv;
+	char *CONST *argv;
 	int ret, ptr, argc;
 
 	argc = (trp -> comm) -> argc;
@@ -1411,8 +1425,8 @@ syntaxtree *trp;
 	if (argc <= 0) return(RET_FAIL);
 	if (argv[0][0] == '[' && !argv[0][1]
 	&& (--argc <= 0 || argv[argc][0] != ']' || argv[argc][1])) {
-		fputs("] missing", stderr);
-		fputnl(stderr);
+		Xfputs("] missing", Xstderr);
+		VOID_C fputnl(Xstderr);
 		return(RET_NOTICE);
 	}
 	if (argc <= 1) return(RET_FAIL);
@@ -1422,19 +1436,20 @@ syntaxtree *trp;
 	if (ret < 0) {
 		switch (ret) {
 			case -1:
-				fputs("argument expected", stderr);
+				Xfputs("argument expected", Xstderr);
 				break;
 			case -2:
-				fprintf2(stderr, "unknown operator %k",
+				fprintf2(Xstderr, "unknown operator %k",
 					argv[ptr]);
 				break;
 			default:
-				fputs("syntax error", stderr);
+				Xfputs("syntax error", Xstderr);
 				break;
 		}
-		fputnl(stderr);
+		VOID_C fputnl(Xstderr);
 		return(RET_NOTICE);
 	}
+
 	return(ret);
 }
 
@@ -1461,7 +1476,7 @@ syntaxtree *trp;
 # endif
 	if (flag == 'V') {
 		for (i = n; i < argc; i++) {
-			if (typeone(argv[i], stdout) >= 0) {
+			if (typeone(argv[i], Xstdout) >= 0) {
 # ifdef	BASHSTYLE
 				ret = RET_SUCCESS;
 # endif
@@ -1487,14 +1502,14 @@ syntaxtree *trp;
 # endif
 				if ((type & CM_NOTFOUND)
 				|| Xaccess(cp, X_OK) < 0) {
-					execerror(cp, ER_COMNOFOUND, 0);
+					execerror(argv, cp, ER_COMNOFOUND, 1);
 					cp = NULL;
 				}
 			}
 
 			if (cp) {
-				kanjifputs(cp, stdout);
-				fputnl(stdout);
+				kanjifputs(cp, Xstdout);
+				VOID_C fputnl(Xstdout);
 # ifdef	BASHSTYLE
 				ret = RET_SUCCESS;
 # endif
@@ -1509,10 +1524,10 @@ syntaxtree *trp;
 	type = checktype(argv[n], &id, 0, 0);
 #endif
 	if (verboseexec) {
-		fprintf2(stderr, "+ %k", argv[n]);
+		fprintf2(Xstderr, "+ %k", argv[n]);
 		for (i = n + 1; i < argc; i++)
-			fprintf2(stderr, " %k", argv[i]);
-		fputnl(stderr);
+			fprintf2(Xstderr, " %k", argv[i]);
+		VOID_C fputnl(Xstderr);
 	}
 
 	(trp -> comm) -> argc -= n;
@@ -1524,18 +1539,19 @@ syntaxtree *trp;
 		ret = exec_simplecom(trp, type, id, 0);
 #endif
 	else {
-		path = strdup2(getconstvar("PATH"));
-		setenv2("PATH", DEFPATH, 1);
+		path = strdup2(getconstvar(ENVPATH));
+		setenv2(ENVPATH, DEFPATH, 1);
 #if	MSDOS
 		ret = exec_simplecom(trp, type, id);
 #else
 		ret = exec_simplecom(trp, type, id, 0);
 #endif
-		setenv2("PATH", path, 1);
-		if (path) free(path);
+		setenv2(ENVPATH, path, 1);
+		free2(path);
 	}
 	(trp -> comm) -> argc = argc;
 	(trp -> comm) -> argv = argv;
+
 	return(ret);
 }
 
@@ -1550,11 +1566,11 @@ syntaxtree *trp;
 	argv = (trp -> comm) -> argv;
 
 	if (argc <= 2) {
-		execerror(NULL, ER_MISSARG, 0);
+		execerror(argv, NULL, ER_MISSARG, 1);
 		return(RET_FAIL);
 	}
 	if (identcheck(argv[2], '\0') <= 0) {
-		execerror(argv[2], ER_NOTIDENT, 0);
+		execerror(argv, argv[2], ER_NOTIDENT, 0);
 		return(RET_FAIL);
 	}
 
@@ -1607,7 +1623,8 @@ syntaxtree *trp;
 			buf[1] = '\0';
 			n = '?';
 			if (quiet) posixoptarg = buf;
-			else execerror(buf, ER_BADOPTIONS, 2);
+			else execerror((trp -> comm) -> argv,
+				buf, ER_BADOPTIONS, 2);
 		}
 		else if (*(cp + 1) == ':') {
 			if (argv[posixoptind][ptr])
@@ -1623,15 +1640,16 @@ syntaxtree *trp;
 				}
 				else {
 					n = '?';
-					execerror(buf, ER_MISSARG, 2);
+					execerror((trp -> comm) -> argv,
+						buf, ER_MISSARG, 2);
 				}
 			}
 			ptr = 0;
 		}
 	}
 
-	if (posixoptarg) setenv2("OPTARG", posixoptarg, 0);
-	else if (ret == RET_SUCCESS) unset("OPTARG", sizeof("OPTARG") - 1);
+	if (posixoptarg) setenv2(ENVOPTARG, posixoptarg, 0);
+	else if (ret == RET_SUCCESS) unset(ENVOPTARG, strsize(ENVOPTARG));
 
 	buf[0] = n;
 	buf[1] = '\0';
@@ -1639,10 +1657,10 @@ syntaxtree *trp;
 
 	n = posixoptind;
 	snprintf2(buf, sizeof(buf), "%d", n);
-	setenv2("OPTIND", buf, 0);
+	setenv2(ENVOPTIND, buf, 0);
 	posixoptind = n;
 
 	return(ret);
 }
 #endif	/* NOPOSIXUTIL */
-#endif	/* !MINIMUMSHELL && (!FD || (FD >= 2 && !_NOORIGSHELL)) */
+#endif	/* DEP_ORIGSHELL && !MINIMUMSHELL */
