@@ -1,39 +1,31 @@
 /*
  *	tree.c
  *
- *	Tree Mode Module
+ *	tree mode module
  */
 
 #include "fd.h"
 #include "func.h"
 #include "kanji.h"
 
+#define	DIRFIELD		3
+#define	TREEFIELD		(((dircountlimit > 0) \
+				? (n_column * 3) / 5 : n_column) - 2)
+#define	FILEFIELD		((dircountlimit > 0) \
+				? (n_column * 2) / 5 - 3 : 0)
+#define	bufptr(y)		(&(tr_scr[(y - 1) * (TREEFIELD + 1)]))
+
 #ifndef	_NOTREE
 
-#if	MSDOS && !defined (_NODOSDRIVE)
-extern int preparedrv __P_((int));
-#endif
-
-
 extern char fullpath[];
+extern int autoupdate;
 extern int win_x;
 extern int win_y;
-#ifndef	_NODOSDRIVE
-extern int lastdrv;
-#endif
-extern int tradlayout;
-extern int sizeinfo;
 
-#define	DIRFIELD	3
-#define	TREEFIELD	(((dircountlimit > 0) \
-			? (n_column * 3) / 5 : n_column) - 2)
-#define	FILEFIELD	((dircountlimit > 0) ? (n_column * 2) / 5 - 3 : 0)
-#define	bufptr(y)	(&(tr_scr[(y - LFILETOP - 1) * (TREEFIELD + 1)]))
-
-static int NEAR evaldir __P_((char *, int));
-static treelist *NEAR maketree __P_((char *, treelist *, treelist *,
+static int NEAR evaldir __P_((CONST char *, int));
+static treelist *NEAR maketree __P_((CONST char *, treelist *, treelist *,
 		int, int *));
-static int NEAR _showtree __P_((treelist *, int, int, int));
+static int NEAR _showtree __P_((treelist *, int, int, int, int));
 static VOID NEAR showtree __P_((VOID_A));
 static VOID NEAR treebar __P_((VOID_A));
 static treelist *NEAR _searchtree __P_((treelist *, int, int));
@@ -61,41 +53,42 @@ static treelist *tr_cur = NULL;
 
 
 static int NEAR evaldir(dir, disp)
-char *dir;
+CONST char *dir;
 int disp;
 {
 	DIR *dirp;
 	struct dirent *dp;
 	struct stat st;
 	char path[MAXPATHLEN];
-	int i, x, y, w, len, limit;
+	int i, x, y, w, min, len, limit;
 #if	MSDOS
 	char *cp;
 #endif
 
 	if ((limit = dircountlimit) <= 0) return(1);
 
-	if (disp) for (i = LFILETOP + 1; i < LFILEBOTTOM; i++) {
-		locate(TREEFIELD + 2, i);
-		putch2('|');
-		putterm(l_clear);
+	min = filetop(win);
+	if (disp) for (i = 1; i < FILEPERROW; i++) {
+		Xlocate(TREEFIELD + 2, min + i);
+		Xputch2('|');
+		Xputterm(L_CLEAR);
 	}
-	strcpy(path, dir);
+	strcpy2(path, dir);
 #if	MSDOS
 	cp = path;
 	if (_dospath(cp)) cp += 2;
-	if (strcmp(cp, _SS_)) strcatdelim(path);
+	if (!isrootpath(cp)) strcatdelim(path);
 #else
-	if (strcmp(path, _SS_)) strcatdelim(path);
+	if (!isrootpath(path)) strcatdelim(path);
 #endif
 	len = strlen(path);
 	i = x = 0;
-	y = LFILETOP + 1;
+	y = 1;
 	w = FILEFIELD / 2;
 	if (!(dirp = Xopendir(dir))) return(0);
 	while ((dp = Xreaddir(dirp))) {
 		if (isdotdir(dp -> d_name)) continue;
-		strcpy(&(path[len]), dp -> d_name);
+		strcpy2(&(path[len]), dp -> d_name);
 		if (limit-- <= 0 || (stat2(path, &st) >= 0 && s_isdir(&st))) {
 			if (!disp) {
 				i++;
@@ -103,44 +96,49 @@ int disp;
 			}
 		}
 		else if (disp) {
-			locate(x + TREEFIELD + 4, y);
-			cprintf2("%-*.*k", w, w, dp -> d_name);
+			Xlocate(x + TREEFIELD + 4, min + y);
+			cputstr(w, dp -> d_name);
 			i++;
-			if (++y >= LFILEBOTTOM) {
-				y = LFILETOP + 1;
+			if (++y >= FILEPERROW) {
+				y = 1;
 				x += w + 1;
 				if (x >= w * 2 + 2) break;
 			}
 		}
 	}
 	if (disp && !i) {
-		locate(x + TREEFIELD + 4, LFILETOP + 1);
-		cprintf2("%-*.*s", w, w, "[No Files]");
+		Xlocate(x + TREEFIELD + 4, min + 1);
+		cputstr(w, "[No Files]");
 	}
 	Xclosedir(dirp);
+
 	return(i);
 }
 
 /*ARGSUSED*/
 static treelist *NEAR maketree(path, list, parent, level, maxp)
-char *path;
+CONST char *path;
 treelist *list, *parent;
 int level, *maxp;
 {
+#ifdef	DEP_DOSEMU
+	char tmp[MAXPATHLEN];
+#endif
+#ifndef	NODIRLOOP
+	treelist *lp;
+#endif
 	DIR *dirp;
 	struct dirent *dp;
 	struct stat st;
-	char *cp, *dir, *subdir, cwd[MAXPATHLEN];
-#ifdef	_USEDOSEMU
-	char tmp[MAXPATHLEN];
-#endif
+	CONST char *cp, *subdir;
+	char *dir, cwd[MAXPATHLEN];
 	int i, len;
 
 	if ((level + 1) * DIRFIELD + 2 > TREEFIELD) {
 		*maxp = -2;
 		return(NULL);
 	}
-	if (intrkey()) {
+	if (intrkey(K_ESC)) {
 		*maxp = -1;
 		return(NULL);
 	}
@@ -149,41 +147,46 @@ int level, *maxp;
 		return(NULL);
 	}
 
-#if	MSDOS
-	if (_dospath(path)) path += 2;
+#ifdef	DEP_DOSPATH
+	if (_dospath(path)) len = 2;
+	else
 #endif
-	if (*path == _SC_) {
-		dir = strdup2(_SS_);
-		subdir = path + 1;
-		if (!*subdir) subdir = NULL;
-	}
-#ifdef	_USEDOSEMU
-	else if (_dospath(path)) {
-		dir = strndup2(path, 3);
-		subdir = path + 3;
-		if (!*subdir) subdir = NULL;
-	}
+#ifdef	DOUBLESLASH
+	if ((len = isdslash(path))) /*EMPTY*/;
+	else
 #endif
-	else {
-		len = (cp = strdelim(path, 0)) ? cp - path : strlen(path);
-		dir = strndup2(path, len);
-		subdir = (cp) ? cp + 1 : NULL;
-	}
+#ifdef	DEP_URLPATH
+	if ((len = _urlpath(path, NULL, NULL))) /*EMPTY*/;
+	else
+#endif
+	len = 0;
+	cp = &(path[len]);
+
+	if (*cp == _SC_) cp++;
+	else cp = strdelim(path, 0);
+	len = (cp) ? cp - path : strlen(path);
+
+	dir = strndup2(path, len);
+	subdir = &(path[len]);
+	if (*subdir == _SC_) subdir++;
+	else if (!*subdir) subdir = NULL;
 
 	if (!subdir) len = 0;
-	else len = (cp = strdelim(subdir, 0)) ? cp - subdir : strlen(subdir);
+	else if ((cp = strdelim(subdir, 0))) len = cp - subdir;
+	else len = strlen(subdir);
 
 	*maxp = 0;
 	i = _chdir2(dir);
-	free(dir);
-	if (i < 0 || !(dirp = Xopendir("."))) return(NULL);
+	free2(dir);
+	if (i < 0 || !(dirp = Xopendir(curpath))) return(NULL);
 
 	i = 0;
 	while ((dp = Xreaddir(dirp))) {
-		if (isdotdir(dp -> d_name)
-		|| Xstat(nodospath(tmp, dp -> d_name), &st) < 0
+		if (isdotdir(dp -> d_name)) continue;
+		else if (Xstat(nodospath(tmp, dp -> d_name), &st) < 0
 		|| !s_isdir(&st))
 			continue;
+
 		list = b_realloc(list, *maxp, treelist);
 		if (!subdir) {
 			list[*maxp].name = strdup2(dp -> d_name);
@@ -197,7 +200,7 @@ int level, *maxp;
 			(*maxp)++;
 		}
 		else if (!strnpathcmp(dp -> d_name, subdir, len)
-		&& strlen(dp -> d_name) == len) {
+		&& !(dp -> d_name[len])) {
 			list[0].name = strdup2(dp -> d_name);
 			list[0].sub = &(list[0]);
 			list[0].max = 0;
@@ -234,11 +237,14 @@ int level, *maxp;
 		}
 	}
 	Xclosedir(dirp);
+
 	if (list) for (i = 0; i < *maxp; i++) {
 #ifndef	NODIRLOOP
-		treelist *lp;
-
 		if (!(list[i].name)) lp = NULL;
+# ifdef	DEP_URLPATH
+		else if (list[i].dev == (dev_t)-1 || list[i].ino == (ino_t)-1)
+			lp = NULL;
+# endif
 		else for (lp = parent; lp; lp = lp -> parent)
 			if (lp -> dev == list[i].dev
 			&& lp -> ino == list[i].ino)
@@ -248,7 +254,7 @@ int level, *maxp;
 			list[i].max = 0;
 		}
 		else
-#endif
+#endif	/* !NODIRLOOP */
 		if (list[i].sub)
 			list[i].sub = maketree(subdir, NULL, &(list[i]),
 				level + 1, &(list[i].max));
@@ -263,21 +269,22 @@ int level, *maxp;
 #if	MSDOS
 	if (_dospath(path)) path += 2;
 #endif
-#if	MSDOS || defined (_NODOSDRIVE)
-	if (*path == _SC_)
-#else
+#ifdef	DEP_DOSEMU
 	if (*path == _SC_ || _dospath(path))
+#else
+	if (*path == _SC_)
 #endif
 	{
 		if (_chdir2(fullpath) < 0) error(fullpath);
 	}
-	else if (strcmp(path, ".") && _chdir2(cwd) < 0) error("..");
+	else if (isdotdir(path) != 2 && _chdir2(cwd) < 0) error(parentpath);
+
 	return(list);
 }
 
-static int NEAR _showtree(list, max, nest, y)
+static int NEAR _showtree(list, max, nest, min, y)
 treelist *list;
-int max, nest, y;
+int max, nest, min, y;
 {
 	char *cp;
 	int i, j, w, tmp;
@@ -286,11 +293,12 @@ int max, nest, y;
 	tmp = y;
 	for (i = 0; i < max; i++) {
 		if (nest > 0) {
-			for (j = tmp; j < y; j++)
-			if (j > LFILETOP && j < LFILEBOTTOM)
+			for (j = tmp; j < y; j++) {
+				if (j <= 0 || j >= FILEPERROW) continue;
 				*(bufptr(j) + (nest - 1) * DIRFIELD) = '|';
+			}
 		}
-		if (y > LFILETOP && y < LFILEBOTTOM) {
+		if (y > 0 && y < FILEPERROW) {
 			cp = bufptr(y);
 			if (nest > 0)
 				memcpy(&(cp[(nest - 1) * DIRFIELD]), "+--", 3);
@@ -310,29 +318,32 @@ int max, nest, y;
 		y++;
 		tmp = y;
 		if (w >= DIRFIELD && list[i].sub)
-			y = _showtree(list[i].sub, list[i].max, nest + 1, y);
+			y = _showtree(list[i].sub, list[i].max,
+				nest + 1, min, y);
 	}
+
 	return(y);
 }
 
 static VOID NEAR showtree(VOID_A)
 {
-	int i;
+	int i, min;
 
-	for (i = LFILETOP; i <= LFILEBOTTOM; i++) {
-		locate(0, i);
-		putterm(l_clear);
+	min = filetop(win);
+	for (i = 0; i < FILEPERROW; i++) {
+		Xlocate(0, min + i);
+		Xputterm(L_CLEAR);
 	}
-	for (i = LFILETOP + 1; i < LFILEBOTTOM; i++) {
+	for (i = 1; i < FILEPERROW; i++) {
 		memset(bufptr(i), ' ', TREEFIELD);
 		bufptr(i)[TREEFIELD] = '\0';
 	}
-	_showtree(tr_root -> sub, 1, 0, tr_top);
-	for (i = LFILETOP + 1; i < LFILEBOTTOM; i++) {
-		locate(1, i);
-		if (i == tr_line) putterm(t_standout);
-		cprintf2("%-*.*k", TREEFIELD, TREEFIELD, bufptr(i));
-		if (i == tr_line) putterm(end_standout);
+	_showtree(tr_root -> sub, 1, 0, min, tr_top - min);
+	for (i = 1; i < FILEPERROW; i++) {
+		Xlocate(1, min + i);
+		if (min + i == tr_line) Xputterm(T_STANDOUT);
+		cputstr(TREEFIELD, bufptr(i));
+		if (min + i == tr_line) Xputterm(END_STANDOUT);
 	}
 	evaldir(treepath, 1);
 	keyflush();
@@ -340,10 +351,10 @@ static VOID NEAR showtree(VOID_A)
 
 static VOID NEAR treebar(VOID_A)
 {
-	locate(1, LFILETOP);
-	cprintf2("Tree=%-*.*k", n_column - 6, n_column - 6, treepath);
-	locate(0, L_MESLINE);
-	putterm(l_clear);
+	Xlocate(1, filetop(win));
+	Xcprintf2("Tree=%-*.*k", n_column - 6, n_column - 6, treepath);
+	Xlocate(0, L_MESLINE);
+	Xputterm(L_CLEAR);
 }
 
 VOID rewritetree(VOID_A)
@@ -352,7 +363,7 @@ VOID rewritetree(VOID_A)
 	searchtree();
 	showtree();
 	treebar();
-	tflush();
+	Xtflush();
 }
 
 static treelist *NEAR _searchtree(list, max, nest)
@@ -368,7 +379,7 @@ int max, nest;
 		if (tr_bottom == tr_line) {
 			lp = tr_root;
 			tr_no = i;
-			if (list[i].name) strcpy(treepath, list[i].name);
+			if (list[i].name) strcpy2(treepath, list[i].name);
 			else *treepath = '\0';
 		}
 		tr_bottom++;
@@ -378,31 +389,16 @@ int max, nest;
 			if (tmplp) {
 				lp = (tmplp == tr_root) ? &(list[i]) : tmplp;
 				len = strlen(list[i].name);
-#if	MSDOS
-				if (!_dospath(list[i].name)
-				|| list[i].name[2] != _SC_)
-					len++;
-				memmove(&(treepath[len]), treepath,
+				if (len > 0 && list[i].name[len - 1] == _SC_)
+					len--;
+				memmove(&(treepath[len + 1]), treepath,
 					strlen(treepath) + 1);
-				memcpy(treepath, list[i].name, len - 1);
-#else	/* !MSDOS */
-# ifdef	_NODOSDRIVE
-				if (*list[i].name != _SC_) len++;
-# else
-				if (*list[i].name != _SC_
-				&& !_dospath(list[i].name))
-					len++;
-# endif
-				memmove(&(treepath[len]), treepath,
-					strlen(treepath) + 1);
-				if (*list[i].name != _SC_)
-					memcpy(treepath, list[i].name,
-						len - 1);
-#endif	/* !MSDOS */
-				treepath[len - 1] = _SC_;
+				memcpy(treepath, list[i].name, len);
+				treepath[len] = _SC_;
 			}
 		}
 	}
+
 	return(lp);
 }
 
@@ -424,11 +420,10 @@ treelist *list;
 		lp = (treelist *)malloc2(sizeof(treelist));
 		memcpy((char *)lp, (char *)&(list -> sub[0]),
 			sizeof(treelist));
-		for (i = 1; i < list -> max; i++)
-			if ((list -> sub[i]).name) {
-				free((list -> sub[i]).name);
-				(list -> sub[i]).name = NULL;
-			}
+		for (i = 1; i < list -> max; i++) {
+			free2(list -> sub[i].name);
+			list -> sub[i].name = NULL;
+		}
 	}
 
 	if (_chdir2(treepath) < 0) {
@@ -436,16 +431,17 @@ treelist *list;
 		list -> max = 0;
 		return(1);
 	}
-	for (cp = treepath, i = 0; (cp = strdelim(cp, 0)); cp++, i++);
-	lptmp = maketree(".", list -> sub, list, i, &(list -> max));
+	for (cp = treepath, i = 0; (cp = strdelim(cp, 0)); cp++, i++)
+		/*EMPTY*/;
+	lptmp = maketree(curpath, list -> sub, list, i, &(list -> max));
 	if (_chdir2(fullpath) < 0) lostcwd(fullpath);
 	if (list -> max < 0) {
 		i = (list -> max < -1) ? 1 : 0;
 		if (!(list -> sub)) list -> max = -1;
 		else {
 			list -> max = 2;
-			(list -> sub[1]).name = NULL;
-			free(lp);
+			list -> sub[1].name = NULL;
+			free2(lp);
 		}
 		return(i);
 	}
@@ -456,16 +452,17 @@ treelist *list;
 			&& !strpathcmp(lp -> name, lptmp[i].name))
 				break;
 		if (i < list -> max) {
-			free(lptmp[i].name);
+			free2(lptmp[i].name);
 			for (; i > 0; i--) memcpy((char *)&(lptmp[i]),
 					(char *)&(lptmp[i - 1]),
 					sizeof(treelist));
 			memcpy((char *)&(lptmp[0]), (char *)lp,
 				sizeof(treelist));
 		}
-		free(lp);
+		free2(lp);
 	}
 	list -> sub = lptmp;
+
 	return(1);
 }
 
@@ -478,47 +475,59 @@ treelist *list;
 	if (!list || list -> max >= 0) return(1);
 	if (!expandtree(list)) return(0);
 	if (!(list -> sub)) return(1);
-	cp = treepath + strlen(treepath);
-	if (cp > treepath + 1) {
+	cp = &(treepath[strlen(treepath)]);
+	if (cp > &(treepath[1])) {
 		strcatdelim(treepath);
 		cp++;
 	}
 	for (i = 0; i < list -> max; i++) {
-		strcpy(cp, (list -> sub[i]).name);
+		strcpy2(cp, list -> sub[i].name);
 		if (!expandall(&(list -> sub[i]))) return(0);
 	}
+
 	return(1);
 }
 
 static int NEAR treeup(VOID_A)
 {
-	if (tr_line > LFILETOP + 1) tr_line--;
-	else if (tr_top <= LFILETOP) tr_top++;
+	int min;
+
+	min = filetop(win);
+	if (tr_line > min + 1) tr_line--;
+	else if (tr_top <= min) tr_top++;
 	else return(-1);
 	searchtree();
+
 	return(0);
 }
 
 static int NEAR treedown(VOID_A)
 {
 	char *cp;
-	int oy, otop;
+	int min, oy, otop;
 
+	min = filetop(win);
 	oy = tr_line;
 	otop = tr_top;
-	if (tr_line < tr_bottom - 1 && tr_line < LFILEBOTTOM - 2)
+	if (tr_line < tr_bottom - 1 && tr_line < min + FILEPERROW - 2)
 		tr_line++;
-	else if (tr_bottom >= LFILEBOTTOM) tr_top--;
+	else if (tr_bottom >= min + FILEPERROW) tr_top--;
 	else return(-1);
 	searchtree();
 
 	if (!tr_cur || !(tr_cur -> sub)
-	|| (tr_cur -> sub[tr_no]).max >= 0
-	|| ((tr_cur -> sub[tr_no]).name))
+	|| tr_cur -> sub[tr_no].max >= 0 || tr_cur -> sub[tr_no].name)
 		return(0);
 
 	waitmes();
-	if ((cp = strrdelim(treepath, 0)) == strdelim(treepath, 0)) cp++;
+	cp = strrdelim(treepath, 0);
+	if (cp == treepath) cp++;
+#ifdef	DOUBLESLASH
+	else if (cp - treepath < isdslash(treepath)) cp++;
+#endif
+#ifdef	DEP_URLPATH
+	else if (cp - treepath < _urlpath(treepath, NULL, NULL)) cp++;
+#endif
 	*cp = '\0';
 	if (!expandtree(tr_cur)) {
 		tr_line = oy;
@@ -528,6 +537,7 @@ static int NEAR treedown(VOID_A)
 	}
 	searchtree();
 	redraw = 1;
+
 	return(0);
 }
 
@@ -539,10 +549,11 @@ int max;
 
 	for (i = n = 0; i < max; i++) {
 		n++;
-		if (list[i].name) free(list[i].name);
+		free2(list[i].name);
 		if (list[i].sub) n += freetree(list[i].sub, list[i].max);
 	}
-	free(list);
+	free2(list);
+
 	return(n);
 }
 
@@ -568,14 +579,15 @@ static int NEAR _tree_input(VOID_A)
 {
 	treelist *old;
 	char *cwd;
-	int ch, tmp, half;
+	int ch, min, tmp, half;
 
 	keyflush();
-	Xgetkey(-1, 0);
-	ch = Xgetkey(1, 0);
-	Xgetkey(-1, 0);
+	Xgetkey(-1, 0, 0);
+	ch = Xgetkey(1, 0, autoupdate);
+	Xgetkey(-1, 0, 0);
 
 	old = tr_cur;
+	min = filetop(win);
 	switch (ch) {
 		case K_UP:
 			treeup();
@@ -585,9 +597,8 @@ static int NEAR _tree_input(VOID_A)
 			break;
 		case K_PPAGE:
 			half = (FILEPERROW - 1) / 2;
-			tmp = LFILETOP + half + 1 - tr_line;
-			if (tr_top + half > LFILETOP + 1)
-				half = LFILETOP - tr_top + 1;
+			tmp = min + half + 1 - tr_line;
+			if (tr_top + half > min + 1) half = min - tr_top + 1;
 			if (half > 0) tr_top += half;
 			for (;tmp > 0; tmp--) if (treedown() < 0) break;
 			tr_line += tmp;
@@ -595,19 +606,19 @@ static int NEAR _tree_input(VOID_A)
 			break;
 		case K_NPAGE:
 			half = (FILEPERROW - 1) / 2;
-			tmp = half + (LFILETOP + half + 1) - tr_line;
+			tmp = half + (min + half + 1) - tr_line;
 			while (tmp-- > 0) if (treedown() < 0) break;
-			tmp = tr_line - (LFILETOP + half + 1);
-			tr_line = LFILETOP + half + 1;
-			if (tr_bottom - tmp < LFILEBOTTOM - 1)
-				tmp = tr_bottom - LFILEBOTTOM + 1;
+			tmp = tr_line - (min + half + 1);
+			tr_line = min + half + 1;
+			if (tr_bottom - tmp < min + FILEPERROW - 1)
+				tmp = tr_bottom - min + FILEPERROW + 1;
 			tr_top -= tmp;
 			searchtree();
 			break;
 		case K_BEG:
 		case K_HOME:
 		case '<':
-			tr_line = tr_top = LFILETOP + 1;
+			tr_line = tr_top = min + 1;
 			searchtree();
 			break;
 		case K_EOL:
@@ -633,7 +644,7 @@ static int NEAR _tree_input(VOID_A)
 			break;
 		case '\t':
 			if (!tr_cur || !(tr_cur -> sub)
-			|| (tr_cur -> sub[tr_no]).max >= 0)
+			|| tr_cur -> sub[tr_no].max >= 0)
 				break;
 			waitmes();
 			expandall(&(tr_cur -> sub[tr_no]));
@@ -642,8 +653,8 @@ static int NEAR _tree_input(VOID_A)
 			break;
 		case K_RIGHT:
 			if (!tr_cur || !(tr_cur -> sub)) break;
-			if ((tr_cur -> sub[tr_no]).max >= 0) {
-				if (!((tr_cur -> sub[tr_no]).sub)) break;
+			if (tr_cur -> sub[tr_no].max >= 0) {
+				if (!(tr_cur -> sub[tr_no].sub)) break;
 				if (treedown() < 0) break;
 			}
 			waitmes();
@@ -652,13 +663,13 @@ static int NEAR _tree_input(VOID_A)
 			redraw = 1;
 			break;
 		case K_LEFT:
-			if (tr_cur && (tr_cur -> sub)
-			&& (tr_cur -> sub[tr_no]).sub) {
-				tmp = freetree((tr_cur -> sub[tr_no]).sub,
-					(tr_cur -> sub[tr_no]).max);
+			if (tr_cur && tr_cur -> sub
+			&& tr_cur -> sub[tr_no].sub) {
+				tmp = freetree(tr_cur -> sub[tr_no].sub,
+					tr_cur -> sub[tr_no].max);
 				tr_bottom -= tmp;
-				(tr_cur -> sub[tr_no]).max = -1;
-				(tr_cur -> sub[tr_no]).sub = NULL;
+				tr_cur -> sub[tr_no].max = -1;
+				tr_cur -> sub[tr_no].sub = NULL;
 				redraw = 1;
 				break;
 			}
@@ -669,18 +680,21 @@ static int NEAR _tree_input(VOID_A)
 			} while (&(tr_cur -> sub[tr_no]) != old);
 			break;
 		case 'l':
-			if (!(cwd = inputstr(LOGD_K, 0, -1, NULL, 1))
+			if (!(cwd = inputstr(LOGD_K, 0, -1, NULL, HST_PATH))
 			|| !*(cwd = evalpath(cwd, 0)))
 				break;
 			if (chdir2(cwd) >= 0) {
-				free(cwd);
+				free2(cwd);
 				break;
 			}
 			warning(-1, cwd);
 			ch = '\0';
-			free(cwd);
+			free2(cwd);
 			break;
 		case K_CTRL('L'):
+#if	FD >= 3
+		case K_TIMEOUT:
+#endif
 			rewritefile(1);
 			break;
 		case K_ESC:
@@ -688,12 +702,12 @@ static int NEAR _tree_input(VOID_A)
 		default:
 			if (!isupper2(ch)) break;
 			if (tr_line == tr_bottom - 1) {
-				tr_line = tr_top = LFILETOP + 1;
+				tr_line = tr_top = min + 1;
 				searchtree();
 			}
 			else do {
 				if (treedown() < 0) break;
-				tmp = toupper2(*((tr_cur -> sub[tr_no]).name));
+				tmp = toupper2(*(tr_cur -> sub[tr_no].name));
 			} while (ch != tmp);
 			break;
 	}
@@ -704,13 +718,13 @@ static int NEAR _tree_input(VOID_A)
 static char *NEAR _tree(VOID_A)
 {
 #ifndef	NODIRLOOP
-	struct stat st;
-# ifndef	_NODOSDRIVE
+# ifdef	DEP_DOSDRIVE
 	char tmp[MAXPATHLEN];
 # endif
-#endif
+	struct stat st;
+#endif	/* !NODIRLOOP */
 	char *cp, *cwd, path[MAXPATHLEN];
-	int ch, oy, otop;
+	int ch, min, len, oy, otop;
 
 	keyflush();
 	waitmes();
@@ -723,22 +737,28 @@ static char *NEAR _tree(VOID_A)
 	tr_root -> parent = NULL;
 #endif
 	treepath = path;
+
+#ifdef	DEP_DOSEMU
+	if (dospath(nullstr, path)) len = 3;
+	else
+#endif
+	{
+#ifdef	DOUBLESLASH
+		if ((len = isdslash(fullpath))) /*EMPTY*/;
+		else
+#endif
+#ifdef	DEP_URLPATH
+		if ((len = _urlpath(fullpath, NULL, NULL))) /*EMPTY*/;
+		else
+#endif
 #if	MSDOS
-	tr_cur[0].name = malloc2(3 + 1);
-	strcpy(path, fullpath);
-	strncpy2(tr_cur[0].name, path, 3);
-#else	/* !MSDOS */
-# ifdef	_NODOSDRIVE
-	strcpy(path, fullpath);
-	tr_cur[0].name = strdup2(_SS_);
-# else
-	if (dospath("", path)) tr_cur[0].name = strndup2(path, 3);
-	else {
-		strcpy(path, fullpath);
-		tr_cur[0].name = strdup2(_SS_);
+		len = 3;
+#else
+		len = 1;
+#endif
+		strcpy2(path, fullpath);
 	}
-# endif
-#endif	/* !MSDOS */
+	tr_cur[0].name = strndup2(path, len);
 
 #ifndef	NODIRLOOP
 	if (Xstat(nodospath(tmp, tr_cur[0].name), &st) < 0)
@@ -753,23 +773,19 @@ static char *NEAR _tree(VOID_A)
 		0, &(tr_cur[0].max));
 
 	tr_line = 0;
-#if	MSDOS
-	cwd = path + 2;
-#else	/* !MSDOS */
-	cwd = path;
-# ifndef	_NODOSDRIVE
-	if (_dospath(cwd)) cwd += 2;
-# endif
-#endif	/* !MSDOS */
-	if (!strcmp(cwd, _SS_)) /*EMPTY*/;
+	if (path[len - 1] == _SC_) len--;
+	cwd = &(path[len]);
+
+	if (isrootpath(cwd)) /*EMPTY*/;
 	else for (cp = cwd; (cp = strdelim(cp, 0)); cp++, tr_line++)
 		if ((tr_line + 1) * DIRFIELD + 2 > TREEFIELD
 		|| !(tr_cur = &(tr_cur -> sub[0])))
 			break;
-	tr_line += (tr_top = LFILETOP + 1);
-	if (tr_line >= LFILEBOTTOM - 2) {
-		tr_top += LFILEBOTTOM - 2 - tr_line;
-		tr_line = LFILEBOTTOM - 2;
+	min = filetop(win);
+	tr_line += (tr_top = min + 1);
+	if (tr_line >= min + FILEPERROW - 2) {
+		tr_top -= tr_line - (min + FILEPERROW - 2);
+		tr_line = min + FILEPERROW - 2;
 	}
 
 	tr_scr = malloc2((FILEPERROW - 1) * (TREEFIELD + 1));
@@ -782,65 +798,73 @@ static char *NEAR _tree(VOID_A)
 		otop = tr_top;
 		redraw = 0;
 		win_y = tr_line;
-		locate(win_x, win_y);
-		tflush();
+		Xlocate(win_x, win_y);
+		Xtflush();
 
 		if ((ch = _tree_input()) == 'l') {
-			free(tr_scr);
+			free2(tr_scr);
 			freetree(tr_root, 1);
 			return(fullpath);
 		}
 
 		if (redraw || tr_top != otop) showtree();
 		else if (oy != tr_line) {
-			locate(1, tr_line);
-			putterm(t_standout);
-			cprintf2("%-*.*k",
-				TREEFIELD, TREEFIELD, bufptr(tr_line));
-			putterm(end_standout);
-			locate(1, oy);
-			if (stable_standout) putterm(end_standout);
-			else cprintf2("%-*.*k",
-				TREEFIELD, TREEFIELD, bufptr(oy));
+			Xlocate(1, tr_line);
+			Xputterm(T_STANDOUT);
+			cputstr(TREEFIELD, bufptr(tr_line - min));
+			Xputterm(END_STANDOUT);
+			Xlocate(1, oy);
+			if (stable_standout) Xputterm(END_STANDOUT);
+			else cputstr(TREEFIELD, bufptr(oy - min));
 			evaldir(path, 1);
 		}
 	} while (ch != K_ESC && ch != K_CR);
 
-	free(tr_scr);
+	free2(tr_scr);
 	freetree(tr_root, 1);
 	treepath = NULL;
 	if (ch == K_ESC) return(NULL);
+
 	return(strdup2(path));
 }
 
 /*ARGSUSED*/
-char *tree(cleanup, ddp)
-int cleanup, *ddp;
+char *tree(cleanup, drvp)
+int cleanup, *drvp;
 {
+#ifdef	DEP_PSEUDOPATH
+	int drv;
+#endif
 	char *path, *dupfullpath;
 
 	if (FILEPERROW < WFILEMINTREE) {
 		warning(0, NOROW_K);
 		return(NULL);
 	}
+#ifdef	DEP_PSEUDOPATH
+	if ((drv = preparedrv(fullpath, NULL, NULL)) < 0) {
+		warning(-1, fullpath);
+		return(NULL);
+	}
+#endif
 	dupfullpath = strdup2(fullpath);
 	do {
 		path = _tree();
 	} while (path == fullpath);
-	if (ddp) {
-#ifndef	_NODOSDRIVE
-# if	MSDOS
-		if (lastdrv < 0) *ddp = preparedrv(dospath(path, NULL));
-		else
-# endif
-		*ddp = lastdrv;
-		lastdrv = -1;
-#endif
+#ifdef	DEP_PSEUDOPATH
+	if (drvp && (*drvp = preparedrv(path, NULL, NULL)) < 0) {
+		warning(-1, path);
+		free2(path);
+		path = NULL;
 	}
+#endif
 	if (chdir2(dupfullpath) < 0) lostcwd(NULL);
-	free(dupfullpath);
-
+	free2(dupfullpath);
+#ifdef	DEP_PSEUDOPATH
+	shutdrv(drv);
+#endif
 	if (cleanup) rewritefile(1);
+
 	return(path);
 }
 #endif	/* !_NOTREE */
