@@ -1,95 +1,59 @@
 /*
  *	unixemu.h
  *
- *	UNIX Emulate Functions
+ *	UNIX function emulation
  */
 
-#ifndef	_UNIXEMU_H
-#define	_UNIXEMU_H
+#if	MSDOS && !defined (__UNIXEMU_H_)
+#define	__UNIXEMU_H_
 
-#ifndef	__SYS_TYPES_STAT_H_
-#define	__SYS_TYPES_STAT_H_
-#include <sys/types.h>
-#include <sys/stat.h>
-#endif
-
-#ifdef	USETIMEH
-#include <time.h>
-#endif
-
-#ifdef	MAXPATHLEN
 #undef	MAXPATHLEN
-#endif
-#define	MAXPATHLEN	260
+#define	MAXPATHLEN		260
 
-#ifdef	MAXNAMLEN
 #undef	MAXNAMLEN
-#endif
-#define	MAXNAMLEN	255
+#define	MAXNAMLEN		255
 
 #ifndef	R_OK
-#define	R_OK		4
+#define	R_OK			4
 #endif
 #ifndef	W_OK
-#define	W_OK		2
+#define	W_OK			2
 #endif
 #ifndef	X_OK
-#define	X_OK		1
+#define	X_OK			1
 #endif
 #ifndef	F_OK
-#define	F_OK		0
+#define	F_OK			0
 #endif
 
 #ifndef	S_IFLNK
-#define	S_IFLNK		0120000
+#define	S_IFLNK			0120000
 #endif
 #ifndef	S_IFSOCK
-#define	S_IFSOCK	0140000
+#define	S_IFSOCK		0140000
 #endif
 #ifndef	S_IFIFO
-#define	S_IFIFO		0010000
+#define	S_IFIFO			0010000
 #endif
 #ifndef	S_ISUID
-#define	S_ISUID		0004000
+#define	S_ISUID			0004000
 #endif
 #ifndef	S_ISGID
-#define	S_ISGID		0002000
+#define	S_ISGID			0002000
 #endif
 #ifndef	S_ISVTX
-#define	S_ISVTX		0001000
-#endif
-
-#if	!defined (DJGPP) || (DJGPP < 2)
-#define	ENOTEMPTY	EACCES
+#define	S_ISVTX			0001000
 #endif
 
 #ifndef	DJGPP
 # ifndef	random
-# define	random()		rand()
+# define	random()	rand()
 # endif
 #define	kill(pid, sig)		((raise(sig)) ? -1 : 0)
-#define	SIGALRM		11	/* SIGSEGV */
+#define	SIGALRM			11	/* SIGSEGV */
 #undef	SIGSEGV
 #undef	SIGTERM
 #undef	SIGILL
-# ifdef	__TURBOC__
-# undef	EFAULT
-# undef	ENFILE
-# undef	ENOSPC
-# undef	EROFS
-# undef	EPERM
-# undef	EINTR
-# undef	EIO
-# undef	ENXIO
-# undef	ENOTDIR
-# undef	EISDIR
-# endif
-# ifndef	EINTR
-# define	EINTR		0
-# endif
-# ifndef	ENOTDIR
-# define	ENOTDIR		ENOENT
-# endif
 #endif	/* !DJGPP */
 
 typedef struct _dirdesc {
@@ -106,9 +70,9 @@ typedef struct _dirdesc {
 	char *dd_path;
 } DIR;
 
-#define	DID_IFNORMAL	000
-#define	DID_IFLABEL	001
-#define	DID_IFLFN	002
+#define	DID_IFNORMAL		000
+#define	DID_IFLABEL		001
+#define	DID_IFLFN		002
 
 struct dirent {
 	long d_off;
@@ -132,4 +96,4 @@ extern int utime __P_((CONST char *, CONST time_t[]));
 extern int utime __P_((CONST char *, CONST struct utimbuf *));
 #endif
 
-#endif	/* !_UNIXEMU_H */
+#endif	/* MSDOS && !__UNIXEMU_H_ */
